#!/usr/bin/env python3
# coding: utf-8
from socket import socket, AF_INET, SOCK_DGRAM
import sys
import threading
import time

import random

NUM_MASTERSERVERS = 4
MASTERSERVER_PORT = 8283


TIMEOUT = 2
BUFFER_SIZE = 4096
NUM_RETRIES =  3


# retries after waiting a specific time if the simple retry method failed.
# SLEEP_SECS is multiplied with 2 on every retry
FORCE_SLEEP = True
SLEEP_SECS = 2.0
NUM_SLEEP_RETRIES = 2

# src/mastersrv/mastersrv.h
PACKET_GETLIST = b"\xff\xff\xff\xffreq2"
PACKET_LIST = b"\xff\xff\xff\xfflis2"

PACKET_GETINFO = b"\xff\xff\xff\xffgie3"
PACKET_INFO = b"\xff\xff\xff\xffinf3"


# see CNetBase::SendControlMsgWithToken
def pack_control_msg_with_token(token_srv,token_cl):
	NET_PACKETFLAG_CONTROL = 1
	NET_CTRLMSG_TOKEN = 5
	NET_TOKENREQUEST_DATASIZE = 512
	b = [0]*(4 + 3 + NET_TOKENREQUEST_DATASIZE)
	# Header
	b[0] = (NET_PACKETFLAG_CONTROL<<2)&0xfc
	b[3] = (token_srv >> 24) & 0xff
	b[4] = (token_srv >> 16) & 0xff
	b[5] = (token_srv >> 8) & 0xff
	b[6] = (token_srv) & 0xff
	# Data
	b[7] = NET_CTRLMSG_TOKEN
	b[8] = (token_cl >> 24) & 0xff
	b[9] = (token_cl >> 16) & 0xff
	b[10] = (token_cl >> 8) & 0xff
	b[11] = (token_cl) & 0xff
	return bytes(b)


def unpack_control_msg_with_token(msg):
	b = list(msg)
	token_cl = (b[3] << 24) + (b[4] << 16) + (b[5] << 8) + (b[6])
	token_srv = (b[8] << 24) + (b[9] << 16) + (b[10] << 8) + (b[11])
	return token_cl,token_srv


# CNetBase::SendPacketConnless
def header_connless(token_srv, token_cl):
	NET_PACKETFLAG_CONNLESS = 8
	NET_PACKETVERSION = 1
	b = [0]*9
	# Header
	b[0] = ((NET_PACKETFLAG_CONNLESS<<2)&0xfc) | (NET_PACKETVERSION&0x03)
	b[1] = (token_srv >> 24) & 0xff
	b[2] = (token_srv >> 16) & 0xff
	b[3] = (token_srv >> 8) & 0xff
	b[4] = (token_srv) & 0xff
	# ResponseToken
	b[5] = (token_cl >> 24) & 0xff
	b[6] = (token_cl >> 16) & 0xff
	b[7] = (token_cl >> 8) & 0xff
	b[8] = (token_cl) & 0xff
	return bytes(b)


# CVariableInt::Unpack from src/engine/shared/compression.cpp
def unpack_int(b):
	l = list(b[:5])
	i = 0
	Sign = (l[i]>>6)&1
	res = l[i] & 0x3F

	for _ in (0,):
		if not (l[i]&0x80):
			break
		i+=1
		res |= (l[i]&(0x7F))<<(6)

		if not (l[i]&0x80):
			break
		i+=1
		res |= (l[i]&(0x7F))<<(6+7)

		if not (l[i]&0x80):
			break
		i+=1
		res |= (l[i]&(0x7F))<<(6+7+7)

		if not (l[i]&0x80):
			break
		i+=1
		res |= (l[i]&(0x7F))<<(6+7+7+7)

	i += 1
	res ^= -Sign
	return res, b[i:]


class Server_Info(threading.Thread):

	def __init__(self, address):
		self.address = address
		self.info = None
		self.finished = False
		threading.Thread.__init__(self, target = self.run)

	def  __str__(self):
		return str(self.info)
	
	def __getitem__(self, key):
		return self.info[key]

	def run(self):
		self.info = get_server_info(self.address)
		self.finished = True


def get_server_info(address):

	try:
		sock = socket(AF_INET, SOCK_DGRAM)
		sock.settimeout(TIMEOUT)
		
		# function definition
		def send_token(sock, address, timeout=TIMEOUT):
			token = random.randrange(0x100000000)

			# Token request
			sock.sendto(pack_control_msg_with_token(-1,token),address)

			# send and receive
			sock.settimeout(timeout)
			data, _ = sock.recvfrom(BUFFER_SIZE)

			# calculate expected token
			token_cl, token_srv = unpack_control_msg_with_token(data)

			# return, whether the correct token was received
			return token_cl == token, token_cl, token_srv
		

		retries = NUM_RETRIES
		token_success, token_cl, token_srv = send_token(sock, address)

		while not token_success and retries > 0:
			retries -= 1
			token_success, token_cl, token_srv = send_token(sock, address, sock.gettimeout() * 2)
		
		sock.settimeout(TIMEOUT) # reset to default value

		if retries == 0:
			raise ValueError(f"Failed to retrieve token from: {address}")
		
		# function definition
		def send_header(sock, address, token_cl, token_srv, timeout=TIMEOUT, sleep_secs = None):
			# Get info request
			sock.sendto(header_connless(token_srv, token_cl) + PACKET_GETINFO + b'\x00', address)
			sock.settimeout(timeout)
			if isinstance(sleep_secs, int):
				time.sleep(sleep_secs)
			data, _ = sock.recvfrom(BUFFER_SIZE)
			
			head = 	header_connless(token_cl, token_srv) + PACKET_INFO + b'\x00'

			received_head = data[:len(head)] # get header
			data = data[len(head):] # skip header

			return received_head == head, data # we don't need the header
		

		data_success, data = send_header(sock, address, token_cl, token_srv)
		retries = NUM_RETRIES

		while not data_success and retries > 0:
			retries -= 1
			data_success, data = send_header(sock, address, token_cl, token_srv, sock.gettimeout() * 2)

		sock.settimeout(TIMEOUT) # reset to default value

		# fast way didn't work, let's try the slow way
		if FORCE_SLEEP and not data_success and retries == 0:
			retries = NUM_SLEEP_RETRIES
			sleep_secs = SLEEP_SECS / 2.0

			while not data_success and retries > 0:
				retries -= 1
				sleep_secs *= 2
				data_success, data = send_header(sock, address, token_cl, token_srv, sock.gettimeout() * 2.0, sleep_secs=sleep_secs)
			if not data_success and retries == 0:
				raise ValueError(f"Failed to retrieve server info from {address}")
		
		elif not data_success and retries == 0:
			raise ValueError(f"Failed to retrieve server info from {address}")

		slots = data.split(b"\x00", maxsplit=5)

		server_info = {}
		server_info["address"] = address
		server_info["version"] = slots[0].decode()
		server_info["name"] = slots[1].decode()
		server_info["hostname"] = slots[2].decode()
		server_info["map"] = slots[3].decode()
		server_info["gametype"] = slots[4].decode()
		data = slots[5]

		# these integers should fit in one byte each
		server_info["flags"], server_info["skill"] = tuple(data[:2])
		data = data[2:]

		server_info["num_players"], data = unpack_int(data)
		server_info["max_players"], data = unpack_int(data)
		server_info["num_clients"], data = unpack_int(data)
		server_info["max_clients"], data = unpack_int(data)
		server_info["players"] = []

		for _ in range(server_info["num_clients"]):
			player = {}
			slots = data.split(b"\x00", maxsplit=2)
			player["name"] = slots[0].decode()
			player["clan"] = slots[1].decode()
			data = slots[2]
			player["country"], data = unpack_int(data)
			player["score"], data = unpack_int(data)
			player["player"], data = unpack_int(data)
			server_info["players"].append(player)

		return server_info
	except AssertionError as e:
		print(*e.args)
	except OSError as e: # Timeout
		#print('> Server %s did not answer' % (address,))
		pass
	except ValueError as e:
		print(e)
	except Exception as e:
		print(e)
		# print('> Server %s did something wrong here' % (address,))
		# import traceback
		# traceback.print_exc()
	finally:
		sock.close() # socket is closed in any case
	return None


class Master_Server_Info(threading.Thread):

	def __init__(self, address):
		self.address = address
		self.servers = []
		self.finished = False
		threading.Thread.__init__(self, target = self.run)

	def run(self):
		self.servers = get_list(self.address)
		self.finished = True


def get_list(address):
	servers = []
	answer = False

	try:
		sock = socket(AF_INET, SOCK_DGRAM)
		sock.settimeout(TIMEOUT)

		token = random.randrange(0x100000000)

		# Token request
		sock.sendto(pack_control_msg_with_token(-1,token),address)
		data, addr = sock.recvfrom(BUFFER_SIZE)
		token_cl, token_srv = unpack_control_msg_with_token(data)
		assert token_cl == token, "Master %s send wrong token: %d (%d expected)" % (address, token_cl, token)
		answer = True

		# Get list request
		sock.sendto(header_connless(token_srv, token_cl) + PACKET_GETLIST, addr)
		head = 	header_connless(token_cl, token_srv) + PACKET_LIST

		while 1:
			data, addr = sock.recvfrom(BUFFER_SIZE)
			# Header should keep consistent
			assert data[:len(head)] == head, "Master %s list header mismatch: %r != %r (expected)" % (address, data[:len(head)], head)

			data = data[len(head):]
			num_servers = len(data) // 18

			for n in range(0, num_servers):
				# IPv4
				if data[n*18:n*18+12] == b"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xff\xff":
					ip = ".".join(map(str, data[n*18+12:n*18+16]))
				# IPv6
				else:
					ip = ":".join(map(str, data[n*18:n*18+16]))
				port = ((data[n*18+16])<<8) + data[n*18+17]
				servers += [(ip, port)]

	except AssertionError as e:
		print(*e.args)
	except OSError as e: # Timeout
		if not answer:
			print('> Master %s did not answer' % (address,))
	except Exception as e:
		# import traceback
		# traceback.print_exc()
		print(e)
	finally:
		sock.close()

	return servers


if __name__ == '__main__':
	master_servers = []

	for i in range(1, NUM_MASTERSERVERS+1):
		m = Master_Server_Info(("master%d.teeworlds.com"%i, MASTERSERVER_PORT))
		master_servers.append(m)
		m.start()

	servers = set()

	while len(master_servers) != 0:
		master_servers[0].join()

		if master_servers[0].finished == True:
			if master_servers[0].servers:
				servers.update(master_servers[0].servers)
			del master_servers[0]


	servers_info = []

	print(len(servers), "servers")

	for server in servers:
		s = Server_Info(server)
		servers_info.append(s)
		s.start()

	num_players = 0
	num_clients = 0
	num_botplayers = 0
	num_botspectators = 0

	while len(servers_info) != 0:
		servers_info[0].join()
		
		if servers_info[0].finished == True:
			if servers_info[0].info:
				server_info = servers_info[0].info
				# check num/max validity
				if   server_info["num_players"] > server_info["max_players"] \
					or server_info["num_clients"] > server_info["max_clients"] \
					or server_info["max_players"] > server_info["max_clients"] \
					or server_info["num_players"] < 0 \
					or server_info["num_clients"] < 0 \
					or server_info["max_clients"] < 0 \
					or server_info["max_players"] < 0 \
					or server_info["max_clients"] > 64:
					server_info["bad"] = 'invalid num/max'
					print('> Server %s has %s' % (server_info["address"], server_info["bad"]))
				# check actual purity
				elif server_info["gametype"] in ('DM', 'TDM', 'CTF', 'LMS', 'LTS') \
					and server_info["max_players"] > 16:
					server_info["bad"] = 'too many players for vanilla'
					print('> Server %s has %s' % (server_info["address"], server_info["bad"]))

				else:
					num_players += server_info["num_players"]
					num_clients += server_info["num_clients"]
					for p in servers_info[0].info["players"]:
						if p["player"] == 2:
							num_botplayers += 1
						if p["player"] == 3:
							num_botspectators += 1

			del servers_info[0]


	print('%d players (%d bots) and %d spectators (%d bots)' % (num_players, num_botplayers, num_clients - num_players, num_botspectators))



import sys
if sys.version_info[0] == 2:
	import urllib
	url_lib = urllib
elif sys.version_info[0] == 3:
	import urllib.request
	url_lib = urllib.request

def fetch_file(url):
	print("trying %s" % url)
	try:
		local = dict(url_lib.urlopen(url).info())
		if "Content-Disposition" in local:
			key_name = "Content-Disposition"
		elif "content-disposition" in local:
			key_name = "content-disposition"
		else:
			return False
		local = local[key_name].split("=")[1]
		url_lib.urlretrieve(url, local)
		return local
	except IOError:
		return False



import os, re, sys

alphanum = "0123456789abcdefghijklmnopqrstuvwzyxABCDEFGHIJKLMNOPQRSTUVWXYZ_"
cpp_keywords = ["auto", "const", "double", "float", "int", "short", "struct", "unsigned", # C
"break", "continue", "else", "for", "long", "signed", "switch", "void",
"case", "default", "enum", "goto", "register", "sizeof", "typedef", "volatile",
"char", "do", "extern", "if", "return", "static", "union", "while",

"asm", "dynamic_cast", "namespace", "reinterpret_cast", "try", # C++
"bool", "explicit", "new", "static_cast", "typeid",
"catch", "false", "operator", "template", "typename",
"class", "friend", "private", "this", "using",
"const_cast", "inline", "public", "throw", "virtual",
"delete", "mutable", "protected", "true", "wchar_t"]

allowed_words = []

#allowed_words += ["bitmap_left", "advance", "glyph"] # ft2


allowed_words += ["qsort"] # stdio / stdlib
allowed_words += ["size_t", "cosf", "sinf", "asinf", "acosf", "atanf", "powf", "fabs", "rand", "powf", "fmod", "sqrtf"] # math.h
allowed_words += ["time_t", "time", "strftime", "localtime"] # time.h
allowed_words += [ # system.h
	"int64",
	"dbg_assert", "dbg_msg", "dbg_break", "dbg_logger_stdout", "dbg_logger_debugger", "dbg_logger_file",
	"mem_alloc", "mem_zero", "mem_free", "mem_copy", "mem_move", "mem_comp", "mem_stats", "total_allocations", "allocated",
	"thread_create", "thread_sleep", "lock_wait", "lock_create", "lock_release", "lock_destroy", "swap_endian",
	"io_open", "io_read", "io_read", "io_write", "io_flush", "io_close", "io_seek", "io_skip", "io_tell", "io_length",
	"str_comp", "str_length", "str_quickhash", "str_format", "str_copy", "str_comp_nocase", "str_sanitize", "str_append",
	"str_comp_num", "str_find_nocase", "str_sanitize_strong", "str_uppercase", "str_toint", "str_tofloat",
	"str_utf8_encode", "str_utf8_rewind", "str_utf8_forward", "str_utf8_decode", "str_sanitize_cc", "str_skip_whitespaces",
	"fs_makedir", "fs_listdir", "fs_storage_path", "fs_is_dir",
	"net_init", "net_addr_comp", "net_host_lookup", "net_addr_str", "type", "port", "net_addr_from_str",
	"net_udp_create", "net_udp_send", "net_udp_recv", "net_udp_close", "net_socket_read_wait",
	"net_stats", "sent_bytes", "recv_bytes", "recv_packets", "sent_packets",
	"time_get", "time_freq", "time_timestamp"]
	
allowed_words += ["vec2", "vec3", "vec4", "round", "clamp", "length", "dot", "normalize", "frandom", "mix", "distance", "min",
        "closest_point_on_line", "max", "absolute"] # math.hpp
allowed_words += [  # tl
	"array", "sorted_array", "string",
	"all", "sort", "add", "remove_index", "remove", "delete_all", "set_size",
	"base_ptr", "size", "swap", "empty", "front", "pop_front", "find_binary", "find_linear", "clear", "range", "end", "cstr",
	"partition_linear", "partition_binary"]
allowed_words += ["fx2f", "f2fx"] # fixed point math

def CheckIdentifier(ident):
	return False

class Checker:
	def CheckStart(self, checker, filename):
		pass
	def CheckLine(self, checker, line):
		pass
	def CheckEnd(self, checker):
		pass

class FilenameExtentionChecker(Checker):
	def __init__(self):
		self.allowed = [".cpp", ".h"]
	def CheckStart(self, checker, filename):
		ext = os.path.splitext(filename)[1]
		if not ext in self.allowed:
			checker.Error("file extention '%s' is not allowed" % ext)

class IncludeChecker(Checker):
	def __init__(self):
		self.disallowed_headers = ["stdio.h", "stdlib.h", "string.h", "memory.h"]
	def CheckLine(self, checker, line):
		if "#include" in line:
			include_file = ""
			if '<' in line:
				include_file = line.split('<')[1].split(">")[0]
				#if not "/" in include_file:
				#	checker.Error("%s is not allowed" % include_file)
			elif '"' in line:
				include_file = line.split('"')[1]
			
			#print include_file
			if include_file in self.disallowed_headers:
				checker.Error("%s is not allowed" % include_file)

class HeaderGuardChecker(Checker):
	def CheckStart(self, checker, filename):
		self.check = ".h" in filename
		self.guard = "#ifndef " + filename[4:].replace("/", "_").replace(".hpp", "").replace(".h", "").upper() + "_H"
	def CheckLine(self, checker, line):
		if self.check:
			#if "#" in line:
			self.check = False
			#if not self.check:
			if line.strip() ==  self.guard:
				pass
			else:
				checker.Error("malformed or missing header guard. Should be '%s'" % self.guard)

class CommentChecker(Checker):
	def CheckLine(self, checker, line):
		if line.strip()[-2:] == "*/" and "/*" in line:
			checker.Error("single line multiline comment")

class FileChecker:
	def __init__(self):
		self.checkers = []
		self.checkers += [FilenameExtentionChecker()]
		self.checkers += [HeaderGuardChecker()]
		self.checkers += [IncludeChecker()]
		self.checkers += [CommentChecker()]

	def Error(self, errormessage):
		self.current_errors += [(self.current_line, errormessage)]

	def CheckLine(self, line):
		for c in self.checkers:
			c.CheckLine(self, line)
		return True
	
	def CheckFile(self, filename):
		self.current_file = filename
		self.current_line = 0
		self.current_errors = []
		for c in self.checkers:
			c.CheckStart(self, filename)
			
		for line in file(filename).readlines():
			self.current_line += 1
			if "ignore_check" in line:
				continue
			self.CheckLine(line)

		for c in self.checkers:
			c.CheckEnd(self)
	
	def GetErrors(self):
		return self.current_errors
		
def cstrip(lines):
	d = ""
	for l in lines:
		if "ignore_convention" in l:
			continue
		l = re.sub("^[\t ]*#.*", "", l)
		l = re.sub("//.*", "", l)
		l = re.sub('\".*?\"', '"String"', l) # remove strings
		d += l.strip() + " "
	d = re.sub('\/\*.*?\*\/', "", d) # remove /* */ comments
	d = d.replace("\t", " ") # tab to space
	d = re.sub("  *", " ", d) # remove double spaces
	#d = re.sub("", "", d) # remove /* */ comments
	
	d = d.strip()
	
	# this eats up cases like 'n {'
	i = 1
	while i < len(d)-2:
		if d[i] == ' ':
			if not (d[i-1] in alphanum and d[i+1] in alphanum):
				d = d[:i] + d[i+1:]
		i += 1
	return d

#def stripstrings(data):
#	return re.sub('\".*?\"', 'STRING', data)
	
def get_identifiers(data):
	idents = {}
	data = " "+data+" "
	regexp = re.compile("[^a-zA-Z0-9_][a-zA-Z_][a-zA-Z0-9_]+[^a-zA-Z0-9_]")
	start = 0
	while 1:
		m = regexp.search(data, start)
		
		if m == None:
			break
		start = m.end()-1
		name = data[m.start()+1:m.end()-1]
		if name in idents:
			idents[name] += 1
		else:
			idents[name] = 1
	return idents

grand_total = 0
grand_offenders = 0

gen_html = 1

if gen_html:
	print "<head>"
	print '<link href="/style.css" rel="stylesheet" type="text/css" />'
	print "</head>"
	print "<body>"



	print '<div id="outer">'

	print '<div id="top_left"><div id="top_right"><div id="top_mid">'
	print '<a href="/"><img src="/images/twlogo.png" alt="teeworlds logo" /></a>'
	print '</div></div></div>'

	print '<div id="menu_left"><div id="menu_right"><div id="menu_mid">'
	print '</div></div></div>'

	print '<div id="tlc"><div id="trc"><div id="tb">&nbsp;</div></div></div>'
	print '<div id="lb"><div id="rb"><div id="mid">'
	print '<div id="container">'

	print '<p class="topic_text">'
	print '<h1>Code Refactoring Progress</h1>'
	print '''This is generated by a script that find identifiers in the code
	that doesn't conform to the code standard. Right now it only shows headers
	because they need to be fixed before we can do the rest of the source.
	This is a ROUGH estimate of the progress'''
	print '</p>'

	print '<p class="topic_text">'
	print '<table>'
	#print "<tr><td><b>%</b></td><td><b>#</b></td><td><b>File</b></td><td><b>Offenders</b></td></tr>"

line_order = 1
total_files = 0
complete_files = 0
total_errors = 0

for (root,dirs,files) in os.walk("src"):
	for filename in files:
		filename = os.path.join(root, filename)
		if "/." in filename or "/external/" in filename or "/base/" in filename or "/generated/" in filename:
			continue
		if "src/osxlaunch/client.h" in filename: # ignore this file, ObjC file
			continue
		if "e_config_variables.h" in filename: # ignore config files
			continue
		if "src/game/variables.hpp" in filename: # ignore config files
			continue
			
		if not (".hpp" in filename or ".h" in filename or ".cpp" in filename):
			continue
		
		#total_files += 1
		
		#if not "src/engine/client/ec_client.cpp" in filename:
		#	continue
		
		f = FileChecker()
		f.CheckFile(filename)
		num_errors = len(f.GetErrors())
		total_errors += num_errors
		
		if num_errors:
			print '<tr style="background: #e0e0e0"><td colspan="2">%s, %d errors</td></tr>' % (filename, num_errors),
			for line, msg in f.GetErrors():
				print '<tr"><td>%d</td><td>%s</td></tr>' % (line, msg)
			#print '<table>'
			#GetErrors()
			
		
		
		
		if 0:
			text = cstrip(file(filename).readlines()) # remove all preprocessor stuff and comments
			#text = stripstrings(text) # remove strings (does not solve all cases however)
			#print text
			
			idents = get_identifiers(text)
			offenders = 0
			total = 0
			offender_list = {}
			for name in idents:
				#print name
				if len(name) <= 2: # skip things that are too small
					continue
				if name in cpp_keywords: # skip keywords
					continue
				if name in allowed_words: # skip allowed keywords
					continue
					
				total += idents[name]
				if name != name.lower(): # strip names that are not only lower case
					continue
				offender_list[name] = idents[name]
				if not gen_html:
					print "[%d] %s"%(idents[name], name)
				offenders += idents[name]
			
			grand_total += total
			grand_offenders += offenders
			
			if total == 0:
				total = 1
			
			line_order = -line_order
			
			
			done = int((1-(offenders / float(total))) * 100)
			if done == 100:
				complete_files += 1

			if done != 100 and gen_html:
				color = "#ffa0a0"
				if done > 20:
					color = "#ffd080"
				if done > 50:
					color = "#ffff80"
				if done > 75:
					color = "#e0ff80"
				if done == 100:
					color = "#80ff80"
					
				line_color = "#f0efd5"
				if line_order > 0:
					line_color = "#ffffff"
				
				offender_string = ""
				count = 0
				for name in offender_list:
					count += 1
					offender_string += "[%d]%s " % (offender_list[name], name)
					
					if count%5 == 0:
						offender_string += "<br/>"
					
				print '<tr style="background: %s">' % line_color,
				print '<td style="text-align: right; background: %s"><b>%d%%</b></td><td style="text-align: center">%d</td><td>%s</td>' % (color, done, offenders, filename),
				print '<td style="text-align: right">%s</td>' % offender_string
				print "</tr>"
			count = 0

if gen_html:
	print "</table>"
	
	print "<h1>%d errors</h1>" % total_errors
	
	
	if 0:
		print "<h1>%.1f%% Identifiers done</h1>" % ((1-(grand_offenders / float(grand_total))) * 100)
		print "%d left of %d" % (grand_offenders, grand_total)
		print "<h1>%.1f%% Files done</h1>" % ((complete_files / float(total_files)) * 100)
		print "%d left of %d" % (total_files-complete_files, total_files)

	print "</p>"
	print "<div style='clear:both;'></div>"
	print '</div>'
	print '</div></div></div>'

	print '<div id="blc"><div id="brc"><div id="bb">&nbsp;</div></div></div>'
	print '</div>'

	print "</body>"



import imp, optparse, os, re, shutil, sys, zipfile
os.chdir(os.path.dirname(os.path.realpath(sys.argv[0])) + "/..")
import twlib

arguments = optparse.OptionParser()
arguments.add_option("-b", "--url-bam", default = "http://github.com/matricks/bam/archive/master.zip", help = "URL from which the bam source code will be downloaded")
arguments.add_option("-t", "--url-teeworlds", default = "http://github.com/teeworlds/teeworlds/archive/master.zip", help = "URL from which the teeworlds source code will be downloaded")
arguments.add_option("-r", "--release-type", default = "server_release client_release", help = "Parts of the game which should be builded (for example client_release, debug, server_release or a combination of any of them)")
(options, arguments) = arguments.parse_args()

bam = options.url_bam[7:].split("/")
version_bam = re.search(r"\d\.\d\.\d", bam[len(bam)-1])
if version_bam:
	version_bam = version_bam.group(0)
else:
	version_bam = "trunk"
teeworlds = options.url_teeworlds[7:].split("/")
version_teeworlds = re.search(r"\d\.\d\.\d", teeworlds[len(teeworlds)-1])
if version_teeworlds:
	version_teeworlds = version_teeworlds.group(0)
else:
	version_teeworlds = "trunk"

bam_execution_path = ""
if version_bam < "0.3.0":
	bam_execution_path = "src%s" % os.sep

name = "teeworlds"

flag_clean = True
flag_download = True
flag_unpack = True
flag_build_bam = True
flag_build_teeworlds = True
flag_make_release = True

if os.name == "nt":
	platform = "win32"
else:
	# get name
	osname = os.popen("uname").readline().strip().lower()
	if osname == "darwin":
		osname = "osx"

	# get arch
	machine = os.popen("uname -m").readline().strip().lower()
	arch = "unknown"

	if machine[0] == "i" and machine[2:] == "86":
		arch = "x86"
	elif machine == "x86_64":
		arch = "x86_64"
	elif "power" in machine.lower():
		arch = "ppc"

	platform = osname + "_" + arch

print("%s-%s-%s" % (name, version_teeworlds, platform))

root_dir = os.getcwd() + os.sep
work_dir = root_dir + "scripts/work"

def unzip(filename, where):
	try:
		z = zipfile.ZipFile(filename, "r")
	except:
		return False
	list = "\n"
	for name in z.namelist():
		list += "%s\n" % name
		try: os.makedirs(where+"/"+os.path.dirname(name))
		except: pass

		try:
			f = open(where+"/"+name, "wb")
			f.write(z.read(name))
			f.close()
		except: pass

	z.close()

	directory = "[^/\n]*?/"
	part_1 = "(?<=\n)"
	part_2 = "[^/\n]+(?=\n)"
	regular_expression = r"%s%s" % (part_1, part_2)
	main_directory = re.search(regular_expression, list)
	while main_directory == None:
		part_1 += directory
		regular_expression = r"%s%s" % (part_1, part_2)
		main_directory = re.search(regular_expression, list)
	main_directory = re.search(r".*/", "./%s" % main_directory.group(0))
	return main_directory.group(0)

def bail(reason):
	print(reason)
	os.chdir(work_dir)
	sys.exit(-1)

def clean():
	print("*** cleaning ***")
	try: shutil.rmtree(work_dir)
	except: pass

def file_exists(file):
	try:
		open(file).close()
		return True
	except:
		return False;

def search_object(directory, object):
	directory = os.listdir(directory)
	for entry in directory:
		match = re.search(object, entry)
		if match != None:
			return entry

# clean
if flag_clean:
	clean()

# make dir
try: os.mkdir(work_dir)
except: pass

# change dir
os.chdir(work_dir)

# download
if flag_download:
	print("*** downloading bam source package ***")
	src_package_bam = twlib.fetch_file(options.url_bam)
	if src_package_bam:
		if version_bam == 'trunk':
			version = re.search(r"-[^-]*?([^-]*?)\.[^.]*$", src_package_bam)
			if version:
				version_bam = version.group(1)
	else:
		bail("couldn't find bam source package and couldn't download it")

	print("*** downloading %s source package ***" % name)
	src_package_teeworlds = twlib.fetch_file(options.url_teeworlds)
	if src_package_teeworlds:
		if version_teeworlds == 'trunk':
			version = re.search(r"-[^-]*?([^-]*?)\.[^.]*$", src_package_teeworlds)
			if version:
				version_teeworlds = version.group(1)
	else:
		bail("couldn't find %s source package and couldn't download it" % name)

# unpack
if flag_unpack:
	print("*** unpacking source ***")
	if hasattr(locals(), 'src_package_bam') == False:
		src_package_bam = search_object(work_dir, r"bam.*?\.")
		if not src_package_bam:
			bail("couldn't find bam source package")
	src_dir_bam = unzip(src_package_bam, ".")
	if not src_dir_bam:
		bail("couldn't unpack bam source package")

	if hasattr(locals(), 'src_package_teeworlds') == False:
		src_package_teeworlds = search_object(work_dir, r"teeworlds.*?\.")
		if not src_package_bam:
			bail("couldn't find %s source package" % name)
	src_dir_teeworlds = unzip(src_package_teeworlds, ".")
	if not src_dir_teeworlds:
		bail("couldn't unpack %s source package" % name)

# build bam
if flag_build_bam:
	print("*** building bam ***")
	os.chdir("%s/%s" % (work_dir, src_dir_bam))
	if os.name == "nt":
		if os.system("g++ -v >nul 2>&1") == 0:
			print("using gcc")
			os.system("make_win32_mingw.bat")
		else:
			print("using cl")
			os.system("make_win32_msvc.bat")
		if file_exists("%sbam.exe" % bam_execution_path) == False:
			bail("failed to build bam")
	else:
		os.system("sh make_unix.sh")
		if file_exists("%sbam" % bam_execution_path) == False:
			bail("failed to build bam")
	os.chdir(work_dir)

# build the game
if flag_build_teeworlds:
	print("*** building %s ***" % name)
	if hasattr(locals(), 'src_dir_bam') == False:
		src_dir_bam = search_object(work_dir, r"bam[^\.]*$") + os.sep
		if src_dir_bam:
			directory = src_dir_bam + bam_execution_path
			file = r"bam"
			if os.name == "nt":
				file += r"\.exe"
			if not search_object(directory, file):
				bail("couldn't find bam")
		else:
			bail("couldn't find bam")
	if hasattr(locals(), 'src_dir_teeworlds') == False:
		src_dir_teeworlds = search_object(work_dir, r"teeworlds[^\.]*$")
		if not src_dir_teeworlds:
			bail("couldn't find %s source" % name)
	command = 1
	if os.name == "nt":
		if os.system("g++ -v >nul 2>&1") == 0:
			print("using gcc")
			os.chdir(src_dir_teeworlds)
			command = os.system('"%s\\%s%sbam" %s' % (work_dir, src_dir_bam, bam_execution_path, options.release_type))
		else:
			print("using cl")
			file = open("build.bat", "wb")
			content = "@echo off\n"
			content += "@REM check if we already have the tools in the environment\n"
			content += "if exist \"%VCINSTALLDIR%\" (\ngoto compile\n)\n"
			content += "@REM Check for Visual Studio\n"
			content += "if exist \"%VS100COMNTOOLS%\" (\nset VSPATH=\"%VS100COMNTOOLS%\"\ngoto set_env\n)\n"
			content += "if exist \"%VS90COMNTOOLS%\" (\nset VSPATH=\"%VS90COMNTOOLS%\"\ngoto set_env\n)\n"
			content += "if exist \"%VS80COMNTOOLS%\" (\nset VSPATH=\"%VS80COMNTOOLS%\"\ngoto set_env\n)\n\n"
			content += "echo You need Microsoft Visual Studio 8, 9 or 10 installed\n"
			content += "exit\n"
			content += "@ setup the environment\n"
			content += ":set_env\n"
			content += "call %VSPATH%vsvars32.bat\n\n"
			content += ":compile\n"
			content += 'cd %s\n"%s\\%s%sbam" config\n"%s\\%s%sbam" %s' % (src_dir_teeworlds, work_dir, src_dir_bam, bam_execution_path, work_dir, src_dir_bam, bam_execution_path, options.release_type)
			file.write(content.encode())
			file.close()
			command = os.system("build.bat")
	else:
		os.chdir(src_dir_teeworlds)
		command = os.system("%s/%s%sbam %s" % (work_dir, src_dir_bam, bam_execution_path, options.release_type))
	if command != 0:
		bail("failed to build %s" % name)
	os.chdir(work_dir)

# make release
if flag_make_release:
	print("*** making release ***")
	if hasattr(locals(), 'src_dir_teeworlds') == False:
		src_dir_teeworlds = search_object(work_dir, r"teeworlds[^\.]*$")
		if not src_dir_teeworlds:
			bail("couldn't find %s source" % name)
	os.chdir(src_dir_teeworlds)
	command = '"%s/%s/scripts/make_release.py" %s %s' % (work_dir, src_dir_teeworlds, version_teeworlds, platform)
	if os.name != "nt":
		command = "python %s" % command
	if os.system(command) != 0:
		bail("failed to make a relase of %s" % name)
	final_output = "FAIL"
	for f in os.listdir("."):
		if version_teeworlds in f and platform in f and name in f and (".zip" in f or ".tar.gz" in f):
			final_output = f
	os.chdir(work_dir)
	shutil.copy("%s/%s" % (src_dir_teeworlds, final_output), "../"+final_output)
	os.chdir(root_dir)
	clean()

print("*** all done ***")



import polib
import json
import sys

JSON_KEY_TRANSL="translated strings"
JSON_KEY_AUTHOR="authors"
JSON_KEY_MODIFY="modified by"
JSON_KEY_OR="or"
JSON_KEY_TR="tr"
JSON_KEY_CO="context"

def reconvert(filename, json_filename):
	"""Converts a l10n file.po into a file.json, keeping author info from a previous file.json if possible"""
	po = polib.pofile(open(filename, encoding='utf-8').read())
	translations = []

	for entry in po:
		if entry.msgstr:
			t_entry = {}
			t_entry[JSON_KEY_OR] = entry.msgid
			t_entry[JSON_KEY_TR] = entry.msgstr
			if entry.msgctxt is not None:
				t_entry[JSON_KEY_CO] = entry.msgctxt
			translations.append(t_entry)

	try:
		previous_l10n = json.load(open(json_filename, encoding='utf-8'), strict=False)
		
		# remove tabs from data
		authors = previous_l10n[JSON_KEY_AUTHOR]
		for i, value in enumerate(authors[JSON_KEY_MODIFY]):
			authors[JSON_KEY_MODIFY][i] = value.replace("\t", "    ")

		result = {JSON_KEY_AUTHOR: authors, JSON_KEY_TRANSL: translations}
		print(" extracted author info from " + json_filename)

	except IOError:
		result = {JSON_KEY_AUTHOR: "", JSON_KEY_TRANSL: translations}
		print(" failed to open " + json_filename + ", skipping author info")

	json.dump(
		result,
		open(json_filename, 'w', encoding='utf-8'),
		ensure_ascii=False,
		indent="\t",
		separators=(',', ': '),
		sort_keys=True,
	)

def normalize(filename):
	"""Normalizes a l10n file.po for better version controlling"""
	po = polib.pofile(open(filename).read())
	entries = list(sorted(sorted(po, key=lambda x: x.msgctxt or ""), key=lambda x: x.msgid))
	po.clear()
	for entry in entries:
		po.append(entry)
	po.save(filename)

def decrement_indent(filename):
	"""decrement indent level of file"""
	with open(filename, encoding='utf-8') as f:
		sanitized_json=f.read().replace('\n\t', '\n')	 
	with open(filename, 'w', encoding='utf-8') as f:
		f.write(sanitized_json)

if __name__ == '__main__':
	"""Normalizes then converts a l10n file.po into a file.json, keeping author info from a previous file.json if possible"""
	for filename in sys.argv[1:]:
		assert(len(filename)>len(".po"))
		json_filename = filename[:-3]+".json"
		normalize(filename)
		print(filename + " -> " + filename + " (normalized)")
		reconvert(filename, json_filename)
		decrement_indent(json_filename)
		print(filename + " -> " + json_filename)



from __future__ import with_statement
import struct
import sys
import re

def convert(input, output):
    with open(input, "r") as in_file:
        with open(output, "w") as out_file:
            def build_dic(parts):
                dic = {}

                for part in parts:
                    key, value = part.split('=')

                    try:
                        dic[key] = int(value)
                    except:
                        dic[key] = value

                return dic

            def get_entry(line):
                while line[-1] == "\r" or line[-1] == "\n":
                    line = line[0:-1]
                parts = []

                quote = 0
                part = ""

                for c in line:
                    if c == "\"":
                        quote = 1-quote
                    elif c == " " and not quote:
                        if part:
                            parts.append(part)
                            part = ""
                    else:
                        part += c

                if part:
                    parts.append(part)

                type = parts[0]

                dic = build_dic(parts[1:])

                return type, dic

            def write_int16(val):
                out_file.write(struct.pack('<h', val))

            def write_info(dic):
                write_int16(dic["size"])

            def write_common(dic):
                write_int16(dic["scaleW"])
                write_int16(dic["scaleH"])
                write_int16(dic["lineHeight"])
                write_int16(dic["base"])

            def write_page(dic):
                pass

            def write_chars(dic):
                pass

            def write_char(dic):
                write_int16(dic["x"])
                write_int16(dic["y"])
                write_int16(dic["width"])
                write_int16(dic["height"])
                write_int16(dic["xoffset"])
                write_int16(dic["yoffset"])
                write_int16(dic["xadvance"])

            def write_default_char():
                write_int16(0)
                write_int16(0)
                write_int16(0)
                write_int16(0)
                write_int16(0)
                write_int16(0)
                write_int16(0)

            def write_kernings(dic):
                pass

            def write_kerning(dic):
                write_int16(dic["amount"])

            def write_default_kerning():
                write_int16(0)

            chars = []
            kernings = []
            for i in range(256):
                chars.append(None)
            for i in range(256*256):
                kernings.append(None)

            def save_char(dic):
                if dic["id"] < 256:
                    chars[dic["id"]] = dic

            def save_kerning(dic):
                kernings[dic["first"] + dic["second"]*256] = dic

            write_table = {
                "info": write_info,
                "common": write_common,
                "page": write_page,
                "chars": write_chars,
                "char": save_char,
                "kernings": write_kernings,
                "kerning": save_kerning
            }

            for line in in_file:
                type, dic = get_entry(line)
                
                write_table[type](dic)

            for i in range(256):
                if chars[i]:
                    write_char(chars[i])
                else:
                    write_default_char()

            for i in range(256*256):
                if kernings[i]:
                    write_kerning(kernings[i])
                else:
                    write_default_kerning()

if len(sys.argv) >= 2:
    print "converting..."

    filenames = sys.argv[1:]
    for filename in filenames:
        input = filename
        output = re.sub("fnt$", "tfnt", input)
            
        print "input: %s, output: %s" % (input, output)
        convert(input, output)
    print "done!"
else:
    print "font converter! converts .fnt files to teeworlds .tfnt"
    print "usage: font_converter <input>"



import sys, os

# generate keys.h file
f = file("src/engine/keys.h", "w")

keynames = []
for i in range(0, 512):
	keynames += ["&%d"%i]

print >>f, "#ifndef ENGINE_KEYS_H"
print >>f, "#define ENGINE_KEYS_H"

# KEY_EXECUTE already exists on windows platforms
print >>f, "#if defined(CONF_FAMILY_WINDOWS)"
print >>f, "   #undef KEY_EXECUTE"
print >>f, "#endif"

print >>f, '/* AUTO GENERATED! DO NOT EDIT MANUALLY! */'
print >>f, "enum"
print >>f, "{"

print >>f, "\tKEY_FIRST = 0,"

highestid = 0
for line in open("scripts/SDL_scancode.h"):
	l = line.strip().split("=")
	if len(l) == 2 and "SDL_SCANCODE_" in line:
		key = l[0].strip().replace("SDL_SCANCODE_", "KEY_")

		value = int(l[1].split(",")[0].strip())
		if key[0:2] == "/*":
			continue
		print >>f, "\t%s = %d,"%(key, value)
		
		keynames[value] = key.replace("KEY_", "").lower()
		
		if value > highestid:
			highestid =value

print >>f, "\tKEY_MOUSE_1 = %d,"%(highestid+1); keynames[highestid+1] = "mouse1"
print >>f, "\tKEY_MOUSE_2 = %d,"%(highestid+2); keynames[highestid+2] = "mouse2"
print >>f, "\tKEY_MOUSE_3 = %d,"%(highestid+3); keynames[highestid+3] = "mouse3"
print >>f, "\tKEY_MOUSE_4 = %d,"%(highestid+4); keynames[highestid+4] = "mouse4"
print >>f, "\tKEY_MOUSE_5 = %d,"%(highestid+5); keynames[highestid+5] = "mouse5"
print >>f, "\tKEY_MOUSE_6 = %d,"%(highestid+6); keynames[highestid+6] = "mouse6"
print >>f, "\tKEY_MOUSE_7 = %d,"%(highestid+7); keynames[highestid+7] = "mouse7"
print >>f, "\tKEY_MOUSE_8 = %d,"%(highestid+8); keynames[highestid+8] = "mouse8"
print >>f, "\tKEY_MOUSE_WHEEL_UP = %d,"%(highestid+9); keynames[highestid+9] = "mousewheelup"
print >>f, "\tKEY_MOUSE_WHEEL_DOWN = %d,"%(highestid+10); keynames[highestid+10] = "mousewheeldown"
print >>f, "\tKEY_MOUSE_9 = %d,"%(highestid+11); keynames[highestid+11] = "mouse9"
print >>f, "\tKEY_LAST,"

print >>f, "};"
print >>f, ""
print >>f, "#endif"

# generate keynames.c file
f = file("src/engine/client/keynames.h", "w")
print >>f, '/* AUTO GENERATED! DO NOT EDIT MANUALLY! */'
print >>f, ''
print >>f, '#ifndef KEYS_INCLUDE'
print >>f, '#error do not include this header!'
print >>f, '#endif'
print >>f, ''
print >>f, "#include <string.h>"
print >>f, ""
print >>f, "const char g_aaKeyStrings[512][20] ="
print >>f, "{"
for n in keynames:
	print >>f, '\t"%s",'%n

print >>f, "};"
print >>f, ""

f.close()




import sys

data = file(sys.argv[1], "rb").read()

i = 0
print "unsigned char", sys.argv[2], "[] = {"
print str(ord(data[0])),
for d in data[1:]:
	s = ","+str(ord(d))
	print s,
	i += len(s)+1

	if i >= 70:
		print ""
		i = 0
print ""
print "};"



import sys

GlobalIdCounter = 0
def GetID():
	global GlobalIdCounter
	GlobalIdCounter += 1
	return GlobalIdCounter
def GetUID():
	return "x%d"%GetID()

def FixCasing(Str):
	NewStr = ""
	NextUpperCase = True
	for c in Str:
		if NextUpperCase:
			NextUpperCase = False
			NewStr += c.upper()
		else:
			if c == "_":
				NextUpperCase = True
			else:
				NewStr += c.lower()
	return NewStr

def FormatName(type, name):
	if "*" in type:
		return "m_p" + FixCasing(name)
	if "[]" in type:
		return "m_a" + FixCasing(name)
	return "m_" + FixCasing(name)

class BaseType:
	def __init__(self, type_name):
		self._type_name = type_name
		self._target_name = "INVALID"
		self._id = GetID() # this is used to remember what order the members have in structures etc

	def Identifyer(self): return "x"+str(self._id)
	def TargetName(self): return self._target_name
	def TypeName(self): return self._type_name
	def ID(self): return self._id;

	def EmitDeclaration(self, name):
		return ["%s %s;"%(self.TypeName(), FormatName(self.TypeName(), name))]
	def EmitPreDefinition(self, target_name):
		self._target_name = target_name
		return []
	def EmitDefinition(self, name):
		return []

class MemberType:
	def __init__(self, name, var):
		self.name = name
		self.var = var

class Struct(BaseType):
	def __init__(self, type_name):
		BaseType.__init__(self, type_name)
	def Members(self):
		def sorter(a):
			return a.var.ID()
		m = []
		for name in self.__dict__:
			if name[0] == "_":
				continue
			m += [MemberType(name, self.__dict__[name])]
		try:
			m.sort(key = sorter)
		except:
			for v in m:
				print(v.name, v.var)
			sys.exit(-1)
		return m

	def EmitTypeDeclaration(self, name):
		lines = []
		lines += ["struct " + self.TypeName()]
		lines += ["{"]
		for member in self.Members():
			lines += ["\t"+l for l in member.var.EmitDeclaration(member.name)]
		lines += ["};"]
		return lines

	def EmitPreDefinition(self, target_name):
		BaseType.EmitPreDefinition(self, target_name)
		lines = []
		for member in self.Members():
			lines += member.var.EmitPreDefinition(target_name+"."+member.name)
		return lines
	def EmitDefinition(self, name):
		lines = ["/* %s */ {" % self.TargetName()]
		for member in self.Members():
			lines += ["\t" + " ".join(member.var.EmitDefinition("")) + ","]
		lines += ["}"]
		return lines

class Array(BaseType):
	def __init__(self, type):
		BaseType.__init__(self, type.TypeName())
		self.type = type
		self.items = []
	def Add(self, instance):
		if instance.TypeName() != self.type.TypeName():
			error("bah")
		self.items += [instance]
	def EmitDeclaration(self, name):
		return ["int m_Num%s;"%(FixCasing(name)),
			"%s *%s;"%(self.TypeName(), FormatName("[]", name))]
	def EmitPreDefinition(self, target_name):
		BaseType.EmitPreDefinition(self, target_name)

		lines = []
		i = 0
		for item in self.items:
			lines += item.EmitPreDefinition("%s[%d]"%(self.Identifyer(), i))
			i += 1

		if len(self.items):
			lines += ["static %s %s[] = {"%(self.TypeName(), self.Identifyer())]
			for item in self.items:
				itemlines = item.EmitDefinition("")
				lines += ["\t" + " ".join(itemlines).replace("\t", " ") + ","]
			lines += ["};"]
		else:
			lines += ["static %s *%s = 0;"%(self.TypeName(), self.Identifyer())]

		return lines
	def EmitDefinition(self, name):
		return [str(len(self.items))+","+self.Identifyer()]

# Basic Types

class Int(BaseType):
	def __init__(self, value):
		BaseType.__init__(self, "int")
		self.value = value
	def Set(self, value):
		self.value = value
	def EmitDefinition(self, name):
		return ["%d"%self.value]
		#return ["%d /* %s */"%(self.value, self._target_name)]

class Float(BaseType):
	def __init__(self, value):
		BaseType.__init__(self, "float")
		self.value = value
	def Set(self, value):
		self.value = value
	def EmitDefinition(self, name):
		return ["%ff"%self.value]
		#return ["%d /* %s */"%(self.value, self._target_name)]

class String(BaseType):
	def __init__(self, value):
		BaseType.__init__(self, "const char*")
		self.value = value
	def Set(self, value):
		self.value = value
	def EmitDefinition(self, name):
		return ['"'+self.value+'"']

class Pointer(BaseType):
	def __init__(self, type, target):
		BaseType.__init__(self, "%s*"%type().TypeName())
		self.target = target
	def Set(self, target):
		self.target = target
	def EmitDefinition(self, name):
		return ["&"+self.target.TargetName()]

class TextureHandle(BaseType):
	def __init__(self):
		BaseType.__init__(self, "IGraphics::CTextureHandle")
	def EmitDefinition(self, name):
		return ["IGraphics::CTextureHandle()"]	

class SampleHandle(BaseType):
	def __init__(self):
		BaseType.__init__(self, "ISound::CSampleHandle")
	def EmitDefinition(self, name):
		return ["ISound::CSampleHandle()"]	

# helper functions

def EmitTypeDeclaration(root):
	for l in root().EmitTypeDeclaration(""):
		print(l)

def EmitDefinition(root, name):
	for l in root.EmitPreDefinition(name):
		print(l)
	print("%s %s = " % (root.TypeName(), name))
	for l in root.EmitDefinition(name):
		print(l)
	print(";")

# Network stuff after this

class Object:
	pass

class Enum:
	def __init__(self, name, values):
		self.name = name
		self.values = values

class Flags:
	def __init__(self, name, values):
		self.name = name
		self.values = values

class NetObject:
	def __init__(self, name, variables):
		l = name.split(":")
		self.name = l[0]
		self.base = ""
		if len(l) > 1:
			self.base = l[1]
		self.base_struct_name = "CNetObj_%s" % self.base
		self.struct_name = "CNetObj_%s" % self.name
		self.enum_name = "NETOBJTYPE_%s" % self.name.upper()
		self.variables = variables
	def emit_declaration(self):
		if self.base:
			lines = ["struct %s : public %s"%(self.struct_name,self.base_struct_name), "{"]
		else:
			lines = ["struct %s"%self.struct_name, "{"]
		for v in self.variables:
			lines += ["\t"+line for line in v.emit_declaration()]
		lines += ["};"]
		return lines
	def emit_validate(self):
		lines = ["case %s:" % self.enum_name]
		lines += ["{"]
		lines += ["\t%s *pObj = (%s *)pData;"%(self.struct_name, self.struct_name)]
		lines += ["\tif(sizeof(*pObj) != Size) return -1;"]
		for v in self.variables:
			lines += ["\t"+line for line in v.emit_validate()]
		lines += ["\treturn 0;"]
		lines += ["}"]
		return lines


class NetEvent(NetObject):
	def __init__(self, name, variables):
		NetObject.__init__(self, name, variables)
		self.base_struct_name = "CNetEvent_%s" % self.base
		self.struct_name = "CNetEvent_%s" % self.name
		self.enum_name = "NETEVENTTYPE_%s" % self.name.upper()

class NetMessage(NetObject):
	def __init__(self, name, variables):
		NetObject.__init__(self, name, variables)
		self.base_struct_name = "CNetMsg_%s" % self.base
		self.struct_name = "CNetMsg_%s" % self.name
		self.enum_name = "NETMSGTYPE_%s" % self.name.upper()
	def emit_unpack(self):
		lines = []
		lines += ["case %s:" % self.enum_name]
		lines += ["{"]
		lines += ["\t%s *pMsg = (%s *)m_aMsgData;" % (self.struct_name, self.struct_name)]
		lines += ["\t(void)pMsg;"]
		for v in self.variables:
			lines += ["\t"+line for line in v.emit_unpack()]
		for v in self.variables:
			lines += ["\t"+line for line in v.emit_unpack_check()]
		lines += ["} break;"]
		return lines
	def emit_declaration(self):
		extra = []
		extra += ["\tint MsgID() const { return %s; }" % self.enum_name]
		extra += ["\t"]
		extra += ["\tbool Pack(CMsgPacker *pPacker)"]
		extra += ["\t{"]
		#extra += ["\t\tmsg_pack_start(%s, flags);"%self.enum_name]
		for v in self.variables:
			extra += ["\t\t"+line for line in v.emit_pack()]
		extra += ["\t\treturn pPacker->Error() != 0;"]
		extra += ["\t}"]


		lines = NetObject.emit_declaration(self)
		lines = lines[:-1] + extra + lines[-1:]
		return lines


class NetVariable:
	def __init__(self, name, default=None):
		self.name = name
		self.default = None if default is None else str(default)
	def emit_declaration(self):
		return []
	def emit_validate(self):
		return []
	def emit_pack(self):
		return []
	def emit_unpack(self):
		return []
	def emit_unpack_check(self):
		return []

class NetString(NetVariable):
	def emit_declaration(self):
		return ["const char *%s;"%self.name]
	def emit_unpack(self):
		return ["pMsg->%s = pUnpacker->GetString();" % self.name]
	def emit_pack(self):
		return ["pPacker->AddString(%s, -1);" % self.name]

class NetStringStrict(NetVariable):
	def emit_declaration(self):
		return ["const char *%s;"%self.name]
	def emit_unpack(self):
		return ["pMsg->%s = pUnpacker->GetString(CUnpacker::SANITIZE_CC|CUnpacker::SKIP_START_WHITESPACES);" % self.name]
	def emit_pack(self):
		return ["pPacker->AddString(%s, -1);" % self.name]

class NetIntAny(NetVariable):
	def emit_declaration(self):
		return ["int %s;"%self.name]
	def emit_unpack(self):
		if self.default is None:
			return ["pMsg->%s = pUnpacker->GetInt();" % self.name]
		else:
			return ["pMsg->%s = pUnpacker->GetIntOrDefault(%s);" % (self.name, self.default)]
	def emit_pack(self):
		return ["pPacker->AddInt(%s);" % self.name]

class NetIntRange(NetIntAny):
	def __init__(self, name, min, max, default=None):
		NetIntAny.__init__(self,name,default=default)
		self.min = str(min)
		self.max = str(max)
	def emit_validate(self):
		return ["if(!CheckInt(\"%s\", pObj->%s, %s, %s)) return -1;"%(self.name, self.name, self.min, self.max)]
	def emit_unpack_check(self):
		return ["if(!CheckInt(\"%s\", pMsg->%s, %s, %s)) break;"%(self.name, self.name, self.min, self.max)]

class NetEnum(NetIntRange):
	def __init__(self, name, enum):
		NetIntRange.__init__(self, name, 0, len(enum.values)-1)

class NetFlag(NetIntAny):
	def __init__(self, name, flag):
		NetIntAny.__init__(self, name)
		if len(flag.values) > 0:
			self.mask = "%s_%s" % (flag.name, flag.values[0])
			for i in flag.values[1:]:
				self.mask += "|%s_%s" % (flag.name, i)
		else:
			self.mask = "0"
	def emit_validate(self):
		return ["if(!CheckFlag(\"%s\", pObj->%s, %s)) return -1;"%(self.name, self.name, self.mask)]
	def emit_unpack_check(self):
		return ["if(!CheckFlag(\"%s\", pMsg->%s, %s)) break;"%(self.name, self.name, self.mask)]

class NetBool(NetIntRange):
	def __init__(self, name, default=None):
		default = None if default is None else int(default)
		NetIntRange.__init__(self,name,0,1,default=default)

class NetTick(NetIntRange):
	def __init__(self, name):
		NetIntRange.__init__(self,name,0,'max_int')

class NetArray(NetVariable):
	def __init__(self, var, size):
		self.base_name = var.name
		self.var = var
		self.size = size
		self.name = self.base_name + "[%d]"%self.size
	def emit_declaration(self):
		self.var.name = self.name
		return self.var.emit_declaration()
	def emit_validate(self):
		lines = []
		for i in range(self.size):
			self.var.name = self.base_name + "[%d]"%i
			lines += self.var.emit_validate()
		return lines
	def emit_unpack(self):
		lines = []
		for i in range(self.size):
			self.var.name = self.base_name + "[%d]"%i
			lines += self.var.emit_unpack()
		return lines
	def emit_pack(self):
		lines = []
		for i in range(self.size):
			self.var.name = self.base_name + "[%d]"%i
			lines += self.var.emit_pack()
		return lines
	def emit_unpack_check(self):
		lines = []
		for i in range(self.size):
			self.var.name = self.base_name + "[%d]"%i
			lines += self.var.emit_unpack_check()
		return lines



import os


PATH = "../src/"


def check_file(filename):
	file = open(filename)
	while 1:
		line = file.readline()
		if len(line) == 0:
			break
		if line[0] == "/" or line[0] == "*" or line[0] == "\r" or line[0] == "\n" or line[0] == "\t":
			continue
		if line[:7] == "#ifndef":
			hg = "#ifndef " + ("_".join(filename.split(PATH)[1].split("/"))[:-2]).upper() + "_H"
			if line[:-1] != hg:
				print "Wrong header guard in " + filename
		else:
			print "Missing header guard in " + filename
		break
	file.close()



def check_dir(dir):
	list = os.listdir(dir)
	for file in list:
		if os.path.isdir(dir+file):
			if file != "external" and file != "generated":
				check_dir(dir+file+"/")
		elif file[-2:] == ".h" and file != "keynames.h":
			check_file(dir+file)

check_dir(PATH)



import errno
import subprocess
try:
	from subprocess import DEVNULL
except ImportError:
	import os
	DEVNULL = open(os.devnull, 'wb')
try:
	FileNotFoundError
except NameError:
	FileNotFoundError = OSError
try:
	git_hash = subprocess.check_output(["git", "rev-parse", "--short=16", "HEAD"], stderr=DEVNULL).decode().strip()
	definition = '"{}"'.format(git_hash)
except FileNotFoundError as e:
	if e.errno != errno.ENOENT:
		raise
	definition = "0"
except subprocess.CalledProcessError:
	definition = "0";
print("const char *GIT_SHORTREV_HASH = {};".format(definition))



from collections import namedtuple
import os
import re
import shlex
import subprocess

Config = namedtuple('Config', 'install_name_tool otool verbose')

def dylib_regex(name):
	return re.compile(r'\S*{}\S*'.format(re.escape(name)))

class ChangeDylib:
	def __init__(self, config):
		self.config = config
	def _check_call(self, process_args, *args, **kwargs):
		if self.config.verbose >= 1:
			print("EXECUTING {}".format(" ".join(shlex.quote(x) for x in process_args)))
		if not (self.config.verbose >= 2 and "stdout" not in kwargs):
			kwargs["stdout"] = open(os.devnull, 'wb')
		return subprocess.check_call(process_args, *args, **kwargs)
	def _check_output(self, process_args, *args, **kwargs):
		if self.config.verbose >= 1:
			print("EXECUTING {} FOR OUTPUT".format(" ".join(shlex.quote(x) for x in process_args)))
		return subprocess.check_output(process_args, *args, **kwargs)
	def _install_name_tool(self, *args):
		return self._check_call((self.config.install_name_tool,) + args)
	def _otool(self, *args):
		return self._check_output((self.config.otool,) + args)

	def change(self, filename, from_, to):
		lines = self._otool("-L", filename).decode().splitlines()
		regex = dylib_regex(from_)
		matches = sum([regex.findall(l) for l in lines], [])
		if len(matches) != 1:
			if matches:
				raise ValueError("More than one match found for {}: {}".format(from_, matches))
			else:
				raise ValueError("No matches found for {}".format(from_))
		actual_from = matches[0]
		self._install_name_tool("-change", actual_from, to, filename)

def main():
	import argparse
	p = argparse.ArgumentParser(description="Manipulate shared library dependencies for macOS")

	subcommands = p.add_subparsers(help="Subcommand", dest='command', metavar="COMMAND")
	subcommands.required = True

	change = subcommands.add_parser("change", help="Change a shared library dependency to a given value")
	change.add_argument('-v', '--verbose', action='count', help="Verbose output")
	change.add_argument('--tools', nargs=2, help="Paths to the install_name_tool and otool", default=("install_name_tool", "otool"))
	change.add_argument('filename', metavar="FILE", help="Filename of the executable to manipulate")
	change.add_argument('from_', metavar="FROM", help="Fuzzily matched library name to change")
	change.add_argument('to', metavar="TO", help="Exact name that the library dependency should be changed to")
	args = p.parse_args()

	verbose = args.verbose or 0
	install_name_tool, otool = args.tools
	dylib = ChangeDylib(Config(install_name_tool, otool, verbose))
	dylib.change(filename=args.filename, from_=args.from_, to=args.to)

if __name__ == '__main__':
	main()



from collections import namedtuple
import os
import shlex
import subprocess
import tempfile

ConfigDmgtools = namedtuple('Config', 'dmg hfsplus newfs_hfs verbose')
ConfigHdiutil = namedtuple('Config', 'hdiutil verbose')

def chunks(l, n):
	"""
	Yield successive n-sized chunks from l.
	
	From https://stackoverflow.com/a/312464.
	"""
	for i in range(0, len(l), n):
		yield l[i:i + n]

class Dmg:
	def __init__(self, config):
		self.config = config

	def _check_call(self, process_args, *args, **kwargs):
		if self.config.verbose >= 1:
			print("EXECUTING {}".format(" ".join(shlex.quote(x) for x in process_args)))
		if not (self.config.verbose >= 2 and "stdout" not in kwargs):
			kwargs["stdout"] = open(os.devnull, 'wb')
		subprocess.check_call(process_args, *args, **kwargs)

class Dmgtools(Dmg):
	def _mkfs_hfs(self, *args):
		self._check_call((self.config.newfs_hfs,) + args)
	def _hfs(self, *args):
		self._check_call((self.config.hfsplus,) + args)
	def _dmg(self, *args):
		self._check_call((self.config.dmg,) + args)

	def _create_hfs(self, hfs, volume_name, size):
		if self.config.verbose >= 1:
			print("TRUNCATING {} to {} bytes".format(hfs, size))
		with open(hfs, 'wb') as f:
			f.truncate(size)
		self._mkfs_hfs('-v', volume_name, hfs)

	def _symlink(self, hfs, target, link_name):
		self._hfs(hfs, 'symlink', link_name, target)

	def _add(self, hfs, directory):
		self._hfs(hfs, 'addall', directory)

	def _finish(self, hfs, dmg):
		self._dmg('build', hfs, dmg)

	def create(self, dmg, volume_name, directory, symlinks):
		input_size = sum(os.stat(os.path.join(path, f)).st_size for path, dirs, files in os.walk(directory) for f in files)
		output_size = max(input_size * 2, 1024**2)
		hfs = tempfile.mktemp(prefix=dmg + '.', suffix='.hfs')
		self._create_hfs(hfs, volume_name, output_size)
		self._add(hfs, directory)
		for target, link_name in symlinks:
			self._symlink(hfs, target, link_name)
		self._finish(hfs, dmg)
		if self.config.verbose >= 1:
			print("REMOVING {}".format(hfs))
		os.remove(hfs)

class Hdiutil(Dmg):
	def _hdiutil(self, *args):
		self._check_call((self.config.hdiutil,) + args)

	def create(self, dmg, volume_name, directory, symlinks):
		if symlinks:
			raise NotImplementedError("symlinks are not yet implemented")
		if os.path.exists(volume_name + '.dmg'):
			os.remove(volume_name + '.dmg')
		self._hdiutil('create', '-volname', volume_name, '-srcdir', directory, dmg)

def main():
	import argparse
	p = argparse.ArgumentParser(description="Manipulate dmg archives")

	subcommands = p.add_subparsers(help="Subcommand", dest='command', metavar="COMMAND")
	subcommands.required = True

	create = subcommands.add_parser("create", help="Create a dmg archive from files or directories")
	create.add_argument('-v', '--verbose', action='count', help="Verbose output")
	createx = create.add_mutually_exclusive_group(required=True)
	createx.add_argument('--dmgtools', nargs=3, help="Paths to the dmg and hfsplus executable (https://github.com/mozilla/libdmg-hfsplus) and the newfs_hfs executable (http://pkgs.fedoraproject.org/repo/pkgs/hfsplus-tools/diskdev_cmds-540.1.linux3.tar.gz/0435afc389b919027b69616ad1b05709/diskdev_cmds-540.1.linux3.tar.gz)")
	createx.add_argument('--hdiutil', help="Path to the hdiutil (only exists for macOS at time of writing)")
	create.add_argument('output', metavar="OUTPUT", help="Filename of the output dmg archive")
	create.add_argument('volume_name', metavar="VOLUME_NAME", help="Name of the dmg archive")
	create.add_argument('directory', metavar="DIR", help="Directory to create the archive from")
	create.add_argument('--symlink', metavar="SYMLINK", nargs=2, action="append", help="Symlink the first argument under the second name")
	args = p.parse_args()

	verbose = args.verbose or 0
	symlinks = args.symlink or []
	if args.dmgtools:
		dmg, hfsplus, newfs_hfs = args.dmgtools
		dmg = Dmgtools(ConfigDmgtools(dmg=dmg, hfsplus=hfsplus, newfs_hfs=newfs_hfs, verbose=verbose))
	elif args.hdiutil:
		dmg = Hdiutil(ConfigHdiutil(hdiutil=args.hdiutil, verbose=verbose))
	else:
		raise RuntimeError("unreachable")
	dmg.create(volume_name=args.volume_name, directory=args.directory, dmg=args.output, symlinks=symlinks)

if __name__ == '__main__':
	main()



import shutil, optparse, os, re, sys, zipfile
from distutils.dir_util import copy_tree
os.chdir(os.path.dirname(os.path.realpath(sys.argv[0])) + "/..")
import twlib

arguments = optparse.OptionParser(usage="usage: %prog VERSION PLATFORM [options]\n\nVERSION  - Version number\nPLATFORM - Target platform (f.e. linux_x86, linux_x86_64, osx, src, win32, win64)")
arguments.add_option("-l", "--url-languages", default = "http://github.com/teeworlds/teeworlds-translation/archive/master.zip", help = "URL from which the teeworlds language files will be downloaded")
arguments.add_option("-m", "--url-maps", default = "http://github.com/teeworlds/teeworlds-maps/archive/master.zip", help = "URL from which the teeworlds maps files will be downloaded")
arguments.add_option("-s", "--source-dir", help = "Source directory which is used for building the package")
(options, arguments) = arguments.parse_args()
if len(sys.argv) != 3:
	print("wrong number of arguments")
	print(sys.argv[0], "VERSION PLATFORM")
	sys.exit(-1)
if options.source_dir != None:
	if os.path.exists(options.source_dir) == False:
		print("Source directory " + options.source_dir + " doesn't exist")
		exit(1)
	os.chdir(options.source_dir)

valid_platforms = ["win32", "win64", "osx", "linux_x86", "linux_x86_64", "src"]

name = "teeworlds"
version = sys.argv[1]
platform = sys.argv[2]
exe_ext = ""
use_zip = 0
use_gz = 1
use_dmg = 0
use_bundle = 0
include_data = True
include_exe = True
include_src = False

if not platform in valid_platforms:
	print("not a valid platform")
	print(valid_platforms)
	sys.exit(-1)

if platform == "src":
	include_exe = False
	include_src = True
	use_zip = 1
elif platform == 'win32' or platform == 'win64':
	exe_ext = ".exe"
	use_zip = 1
	use_gz = 0
elif platform == 'osx':
	use_dmg = 1
	use_gz = 0
	use_bundle = 1

def unzip(filename, where):
	try:
		z = zipfile.ZipFile(filename, "r")
	except:
		return False
	for name in z.namelist():
		z.extract(name, where)
	z.close()
	return z.namelist()[0]

def copydir(src, dst, excl=[]):
	for root, dirs, files in os.walk(src, topdown=True):
		if "/." in root or "\\." in root:
			continue
		for name in dirs:
			if name[0] != '.':
				os.mkdir(os.path.join(dst, root, name))
		for name in files:
			if name[0] != '.':
				shutil.copy(os.path.join(root, name), os.path.join(dst, root, name))

def copyfiles(src, dst):
	dir = os.listdir(src)
	for files in dir:
		shutil.copy(os.path.join(src, files), os.path.join(dst, files))

def clean():
	print("*** cleaning ***")
	try:
		shutil.rmtree(package_dir)
		shutil.rmtree(languages_dir)
		shutil.rmtree(maps_dir)
		os.remove(src_package_languages)
		os.remove(src_package_maps)
	except: pass

def shell(cmd):
	if os.system(cmd) != 0:
		clean()
		print("Non zero exit code on: os.system(%s)" % cmd)
		sys.exit(1)

package = "%s-%s-%s" %(name, version, platform)
package_dir = package

source_package_dir = "build/"
if platform == 'win32' or platform == 'linux_x86':
	source_package_dir += "x86/release/"
else:
	source_package_dir += "x86_64/release/"

print("cleaning target")
shutil.rmtree(package_dir, True)
os.mkdir(package_dir)

print("download and extract languages")
src_package_languages = twlib.fetch_file(options.url_languages)
if not src_package_languages:
	print("couldn't download languages")
	sys.exit(-1)
languages_dir = unzip(src_package_languages, ".")
if not languages_dir:
	print("couldn't unzip languages")
	sys.exit(-1)

print("download and extract maps")
src_package_maps = twlib.fetch_file(options.url_maps)
if not src_package_maps:
	print("couldn't download maps")
	sys.exit(-1)
maps_dir = unzip(src_package_maps, ".")
if not maps_dir:
	print("couldn't unzip maps")
	sys.exit(-1)

print("adding files")
shutil.copy("readme.md", package_dir)
shutil.copy("license.txt", package_dir)
shutil.copy("storage.cfg", package_dir)

if include_data and not use_bundle:
	copy_tree(source_package_dir+"data", package_dir+"/data")
	copy_tree(languages_dir, package_dir+"/data/languages")
	copy_tree(maps_dir, package_dir+"/data/maps")
	if platform[:3] == "win":
		shutil.copy("other/config_directory.bat", package_dir)
		shutil.copy(source_package_dir+"SDL2.dll", package_dir)
		shutil.copy(source_package_dir+"freetype.dll", package_dir)

if include_exe and not use_bundle:
	shutil.copy(source_package_dir+name+exe_ext, package_dir)
	shutil.copy(source_package_dir+name+"_srv"+exe_ext, package_dir)
	
if include_src:
	for p in ["src", "scripts", "datasrc", "other", "objs"]:
		os.mkdir(os.path.join(package_dir, p))
		copydir(p, package_dir)
	shutil.copy("bam.lua", package_dir)
	shutil.copy("configure.lua", package_dir)

if use_bundle:
	bins = [name, name+'_srv', 'serverlaunch']
	platforms = ('x86_64')
	for bin in bins:
		to_lipo = []
		for p in platforms:
			fname = bin+'_'+p
			if os.path.isfile(fname):
				to_lipo.append(fname)
		if to_lipo:
			shell("lipo -create -output "+bin+" "+" ".join(to_lipo))

	# create Teeworlds appfolder
	clientbundle_content_dir = os.path.join(package_dir, "Teeworlds.app/Contents")
	clientbundle_bin_dir = os.path.join(clientbundle_content_dir, "MacOS")
	clientbundle_resource_dir = os.path.join(clientbundle_content_dir, "Resources")
	clientbundle_framework_dir = os.path.join(clientbundle_content_dir, "Frameworks")
	binary_path = clientbundle_bin_dir + "/" + name+exe_ext
	freetypelib_path = clientbundle_framework_dir + "/libfreetype.6.dylib"
	os.mkdir(os.path.join(package_dir, "Teeworlds.app"))
	os.mkdir(clientbundle_content_dir)
	os.mkdir(clientbundle_bin_dir)
	os.mkdir(clientbundle_resource_dir)
	os.mkdir(clientbundle_framework_dir)
	copy_tree(source_package_dir+"data", clientbundle_resource_dir+"/data")
	copy_tree(languages_dir, clientbundle_resource_dir+"/data/languages")
	copy_tree(maps_dir, clientbundle_resource_dir+"/data/maps")
	shutil.copy("other/icons/Teeworlds.icns", clientbundle_resource_dir)
	shutil.copy(source_package_dir+name+exe_ext, clientbundle_bin_dir)
	shell("install_name_tool -change /usr/local/opt/freetype/lib/libfreetype.6.dylib @executable_path/../Frameworks/libfreetype.6.dylib " + binary_path)
	shell("install_name_tool -change /usr/local/opt/sdl2/lib/libSDL2-2.0.0.dylib @executable_path/../Frameworks/libSDL2-2.0.0.dylib  " + binary_path)
	shell("cp /usr/local/opt/freetype/lib/libfreetype.6.dylib " + clientbundle_framework_dir)
	shell("cp /usr/local/opt/libpng/lib/libpng16.16.dylib " + clientbundle_framework_dir)
	shell("cp /usr/local/opt/sdl2/lib/libSDL2-2.0.0.dylib " + clientbundle_framework_dir)
	shell("install_name_tool -change /usr/local/opt/libpng/lib/libpng16.16.dylib @executable_path/../Frameworks/libpng16.16.dylib " + freetypelib_path)
	open(os.path.join(clientbundle_content_dir, "Info.plist"), "w").write("""
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
<plist version="1.0">
<dict>
	<key>CFBundleDevelopmentRegion</key>
	<string>English</string>
	<key>CFBundleExecutable</key>
	<string>teeworlds</string>
	<key>CFBundleIconFile</key>
	<string>Teeworlds</string>
	<key>CFBundleInfoDictionaryVersion</key>
	<string>6.0</string>
	<key>CFBundlePackageType</key>
	<string>APPL</string>
	<key>CFBundleSignature</key>
	<string>????</string>
	<key>CFBundleVersion</key>
	<string>%s</string>
	<key>CFBundleIdentifier</key>
	<string>com.TeeworldsClient.app</string>
	<key>NSHighResolutionCapable</key>
	<true/>
</dict>
</plist>
	""" % (version))
	open(os.path.join(clientbundle_content_dir, "PkgInfo"), "w").write("APPL????")

	# create Teeworlds Server appfolder
	serverbundle_content_dir = os.path.join(package_dir, "Teeworlds Server.app/Contents")
	serverbundle_bin_dir = os.path.join(serverbundle_content_dir, "MacOS")
	serverbundle_resource_dir = os.path.join(serverbundle_content_dir, "Resources")
	os.mkdir(os.path.join(package_dir, "Teeworlds Server.app"))
	os.mkdir(serverbundle_content_dir)
	os.mkdir(serverbundle_bin_dir)
	os.mkdir(serverbundle_resource_dir)
	os.mkdir(os.path.join(serverbundle_resource_dir, "data"))
	os.mkdir(os.path.join(serverbundle_resource_dir, "data/maps"))
	os.mkdir(os.path.join(serverbundle_resource_dir, "data/mapres"))
	copy_tree(maps_dir, serverbundle_resource_dir+"/data/maps")
	shutil.copy("other/icons/Teeworlds_srv.icns", serverbundle_resource_dir)
	shutil.copy(source_package_dir+name+"_srv"+exe_ext, serverbundle_bin_dir)
	shutil.copy(source_package_dir+"serverlaunch"+exe_ext, serverbundle_bin_dir + "/"+name+"_server")
	open(os.path.join(serverbundle_content_dir, "Info.plist"), "w").write("""
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
<plist version="1.0">
<dict>
	<key>CFBundleDevelopmentRegion</key>
	<string>English</string>
	<key>CFBundleExecutable</key>
	<string>teeworlds_server</string>
	<key>CFBundleIconFile</key>
	<string>Teeworlds_srv</string>
	<key>CFBundleInfoDictionaryVersion</key>
	<string>6.0</string>
	<key>CFBundlePackageType</key>
	<string>APPL</string>
	<key>CFBundleSignature</key>
	<string>????</string>
	<key>CFBundleVersion</key>
	<string>%s</string>
</dict>
</plist>
	""" % (version))
	open(os.path.join(serverbundle_content_dir, "PkgInfo"), "w").write("APPL????")

if use_zip:
	print("making zip archive")
	zf = zipfile.ZipFile("%s.zip" % package, 'w', zipfile.ZIP_DEFLATED)
	
	for root, dirs, files in os.walk(package_dir, topdown=True):
		for name in files:
			n = os.path.join(root, name)
			zf.write(n, n)
	#zf.printdir()
	zf.close()

if use_gz:
	print("making tar.gz archive")
	shell("tar czf %s.tar.gz %s" % (package, package_dir))

if use_dmg:
	print("making disk image")
	shell("rm -f %s.dmg %s_temp.dmg" % (package, package))
	shell("hdiutil create -srcfolder %s -volname Teeworlds -quiet %s_temp" % (package_dir, package))
	shell("hdiutil convert %s_temp.dmg -format UDBZ -o %s.dmg -quiet" % (package, package))
	shell("rm -f %s_temp.dmg" % package)

clean()

print("done")



import shutil, os, re, sys, zipfile
from distutils.dir_util import copy_tree
os.chdir(os.path.dirname(os.path.realpath(sys.argv[0])) + "/..")
import twlib

def unzip(filename, where):
	try:
		z = zipfile.ZipFile(filename, "r")
	except:
		return False

	# extract files
	for name in z.namelist():
		z.extract(name, where)
	z.close()
	return z.namelist()[0]

def downloadAll(targets):
	version = "6c4af62b8c9853bfca1166d672a16abdbf9f0d26"
	url = "https://github.com/teeworlds/teeworlds-libs/archive/{}.zip".format(version)

	# download and unzip
	src_package_libs = twlib.fetch_file(url)
	if not src_package_libs:
		print("couldn't download libs")
		sys.exit(-1)
	libs_dir = unzip(src_package_libs, ".")
	if not libs_dir:
		print("couldn't unzip libs")
		sys.exit(-1)
	libs_dir = "teeworlds-libs-{}".format(version)

	if "sdl" in targets:
		copy_tree(libs_dir + "/sdl/", "other/sdl/")
	if "freetype" in targets:
		copy_tree(libs_dir + "/freetype/", "other/freetype/")

	# cleanup
	try:
		shutil.rmtree(libs_dir)
		os.remove(src_package_libs)
	except: pass

def main():
    import argparse
    p = argparse.ArgumentParser(description="Download freetype and SDL library and header files for Windows.")
    p.add_argument("targets", metavar="TARGET", nargs='+', choices=["sdl", "freetype"], help='Target to download. Valid choices are "sdl" and "freetype"')
    args = p.parse_args()

    downloadAll(args.targets)

if __name__ == '__main__':
    main()



import os, sys
os.chdir(os.path.dirname(os.path.realpath(sys.argv[0])) + "/..")

notice = [b"/* (c) Magnus Auvinen. See licence.txt in the root of the distribution for more information. */\n", b"/* If you are missing that file, acquire a complete release at teeworlds.com.                */\n"]
exclude = ["src%sengine%sexternal" % (os.sep, os.sep), "src%sosxlaunch" % os.sep]
updated_files = 0

def fix_copyright_notice(filename):
	global updated_files
	f = open(filename, "rb")
	lines = f.readlines()
	f.close()

	i = 0
	length_lines = len(lines)
	if length_lines > 0:
		while i <= length_lines and (lines[i].decode("utf-8").lstrip()[:2] == "//" or lines[i].decode("utf-8").lstrip()[:2] == "/*" and lines[i].decode("utf-8").rstrip()[-2:] == "*/") and ("Magnus" in lines[i].decode("utf-8") or "magnus" in lines[i].decode("utf-8") or "Auvinen" in lines[i].decode("utf-8") or "auvinen" in lines[i].decode("utf-8") or "license" in lines[i].decode("utf-8") or "teeworlds" in lines[i].decode("utf-8")):
			i += 1
	length_notice = len(notice)
	if i > 0:
		j = 0
		while lines[j] == notice[j]:
			j += 1
			if j == length_notice:
				return
		k = j
		j = 0
		while j < length_notice -1 - k:
			lines = [notice[j]] + lines
			j += 1
		while j < length_notice:
			lines[j] = notice[j]
			j += 1
	if length_lines == 0 or i == 0:
		j = length_notice - 1
		while j >= 0:
			lines = [notice[j]] + lines
			j -= 1
	open(filename, "wb").writelines(lines)
	updated_files += 1

skip = False
for root, dirs, files in os.walk("src"):
	for excluding in exclude:
		if root[:len(excluding)] == excluding:
			skip = True
			break
	if skip == True:
		skip = False
		continue
	for name in files:
		filename = os.path.join(root, name)

		if filename[-2:] != ".c" and filename[-4:] != ".cpp" and filename[-2:] != ".h":
			continue

		fix_copyright_notice(filename)

output = "file"
if updated_files != 1:
	output += "s"
print("*** updated %d %s ***" % (updated_files, output))



import json
import polib
import os
import re
import time
import sys

from collections import defaultdict

os.chdir(os.path.dirname(os.path.realpath(sys.argv[0])) + "/..")

format = "{0:40} {1:8} {2:8} {3:8}".format
SOURCE_EXTS = [".c", ".cpp", ".h"]

JSON_KEY_AUTHORS="authors"
JSON_KEY_TRANSL="translated strings"
JSON_KEY_UNTRANSL="needs translation"
JSON_KEY_OLDTRANSL="old translations"
JSON_KEY_CTXT="context"
JSON_KEY_OR="or"
JSON_KEY_TR="tr"

SOURCE_LOCALIZE_RE=re.compile(br'Localize\("(?P<str>([^"\\]|\\.)*)"(, ?"(?P<ctxt>([^"\\]|\\.)*)")?\)')

def parse_source():
	l10n = defaultdict(lambda: [])

	def process_line(line, filename, lineno):
		for match in SOURCE_LOCALIZE_RE.finditer(line):
			str_ = match.group('str').decode()
			ctxt = match.group('ctxt')
			if ctxt is not None:
				ctxt = ctxt.decode()
			l10n[(str_, ctxt)].append((filename, lineno))

	for root, dirs, files in os.walk("src"):
		for name in files:
			filename = os.path.join(root, name)
			
			if os.sep + "external" + os.sep in filename:
				continue

			if os.path.splitext(filename)[1] in SOURCE_EXTS:
				# HACK: Open source as binary file.
				# Necessary some of teeworlds source files
				# aren't utf-8 yet for some reason
				for lineno, line in enumerate(open(filename, 'rb')):
					# let lineno start with 1
					lineno += 1
					# process line
					process_line(line, filename, lineno)

	return l10n

def load_languagefile(filename):
	return json.load(open(filename), strict=False) # accept \t tabs

def write_languagefile(outputfilename, l10n_src, old_l10n_data):
	outputfilename += '.po'

	po = polib.POFile()
	po.metadata = {
		'Project-Id-Version': 'teeworlds-0.7_dev',
		'Report-Msgid-Bugs-To': 'translation@teeworlds.com',
		'POT-Creation-Date': time.strftime("%Y-%m-%d %H:%M%z"),
		'PO-Revision-Date': time.strftime("%Y-%m-%d %H:%M%z"),
		'Language-Team': 'Teeworlds Translations <translation@teeworlds.com>',
		'MIME-Version': '1.0',
		'Content-Type': 'text/plain; charset=utf-8',
		'Content-Transfer-Encoding': '8bit',
	}

	translations = {}
	for type_ in (
		JSON_KEY_OLDTRANSL,
		JSON_KEY_UNTRANSL,
		JSON_KEY_TRANSL,
	):
		if type_ not in old_l10n_data:
			continue
		translations.update({
			(t[JSON_KEY_OR], t.get(JSON_KEY_CTXT)): t[JSON_KEY_TR]
			for t in old_l10n_data[type_]
			if t[JSON_KEY_TR]
		})

	all_items = set(translations) | set(l10n_src)
	tsl_items = set(translations) & set(l10n_src)
	old_items = set(translations) - set(l10n_src)
	new_items = set(l10n_src) - set(translations)

	for msg, ctxt in all_items:
		po.append(polib.POEntry(
			msgid=msg,
			msgctxt=ctxt,
			msgstr=translations.get((msg, ctxt), ""),
			obsolete=(msg in old_items),
			occurrences=l10n_src[msg],
		))
	po.save(outputfilename)

if __name__ == '__main__':
	l10n_src = parse_source()

	po = polib.POFile()
	po.metadata = {
		'Project-Id-Version': 'teeworlds-0.7_dev',
		'Report-Msgid-Bugs-To': 'translation@teeworlds.com',
		'POT-Creation-Date': time.strftime("%Y-%m-%d %H:%M%z"),
		'PO-Revision-Date': 'YEAR-MO-DA HO:MI+ZONE',
		'Language-Team': 'Teeworlds Translations <translation@teeworlds.com>',
		'MIME-Version': '1.0',
		'Content-Type': 'text/plain; charset=utf-8',
		'Content-Transfer-Encoding': '8bit',
	}
	for (msg, ctxt), occurrences in l10n_src.items():
		commenttxt = ctxt
		if(commenttxt):
			commenttxt = 'Context: '+commenttxt
		po.append(polib.POEntry(msgid=msg, msgstr="", occurrences=occurrences, msgctxt=ctxt, comment=commenttxt))
	po.save('datasrc/languages/base.pot')

	for filename in os.listdir("datasrc/languages"):
		try:
			if (os.path.splitext(filename)[1] == ".json"
					and filename != "index.json"):
				filename = "datasrc/languages/" + filename
				write_languagefile(filename, l10n_src, load_languagefile(filename))
		except Exception as e:
			print("Failed on {0}, re-raising for traceback".format(filename))
			raise



import sys
from datatypes import *
import content
import network

def create_enum_table(names, num):
	lines = []
	lines += ["enum", "{"]
	lines += ["\t%s=0,"%names[0]]
	for name in names[1:]:
		lines += ["\t%s,"%name]
	lines += ["\t%s" % num, "};"]
	return lines

def create_flags_table(names):
	lines = []
	lines += ["enum", "{"]
	i = 0
	for name in names:
		lines += ["\t%s = 1<<%d," % (name,i)]
		i += 1
	lines += ["};"]
	return lines

def EmitEnum(names, num):
	print("enum")
	print("{")
	print("\t%s=0," % names[0])
	for name in names[1:]:
		print("\t%s," % name)
	print("\t%s" % num)
	print("};")

def EmitFlags(names, num):
	print("enum")
	print("{")
	i = 0
	for name in names:
		print("\t%s = 1<<%d," % (name,i))
		i += 1
	print("};")

gen_network_header = False
gen_network_source = False
gen_client_content_header = False
gen_client_content_source = False
gen_server_content_header = False
gen_server_content_source = False

if "network_header" in sys.argv: gen_network_header = True
if "network_source" in sys.argv: gen_network_source = True
if "client_content_header" in sys.argv: gen_client_content_header = True
if "client_content_source" in sys.argv: gen_client_content_source = True
if "server_content_header" in sys.argv: gen_server_content_header = True
if "server_content_source" in sys.argv: gen_server_content_source = True

if gen_client_content_header:
	print("#ifndef CLIENT_CONTENT_HEADER")
	print("#define CLIENT_CONTENT_HEADER")

if gen_server_content_header:
	print("#ifndef SERVER_CONTENT_HEADER")
	print("#define SERVER_CONTENT_HEADER")


if gen_client_content_header or gen_server_content_header:
	# print some includes
	print('#include <engine/graphics.h>')
	print('#include <engine/sound.h>')

	# emit the type declarations
	contentlines = open("datasrc/content.py", "rb").readlines()
	order = []
	for line in contentlines:
		line = line.strip()
		if line[:6] == "class ".encode() and "(Struct)".encode() in line:
			order += [line.split()[1].split("(".encode())[0].decode("ascii")]
	for name in order:
		EmitTypeDeclaration(content.__dict__[name])

	# the container pointer
	print('extern CDataContainer *g_pData;')

	# enums
	EmitEnum(["IMAGE_%s"%i.name.value.upper() for i in content.container.images.items], "NUM_IMAGES")
	EmitEnum(["ANIM_%s"%i.name.value.upper() for i in content.container.animations.items], "NUM_ANIMS")
	EmitEnum(["SPRITE_%s"%i.name.value.upper() for i in content.container.sprites.items], "NUM_SPRITES")

if gen_client_content_source or gen_server_content_source:
	if gen_client_content_source:
		print('#include "client_data.h"')
	if gen_server_content_source:
		print('#include "server_data.h"')
	EmitDefinition(content.container, "datacontainer")
	print('CDataContainer *g_pData = &datacontainer;')

# NETWORK
if gen_network_header:

	print("#ifndef GAME_GENERATED_PROTOCOL_H")
	print("#define GAME_GENERATED_PROTOCOL_H")
	print(network.RawHeader)

	for e in network.Enums:
		for l in create_enum_table(["%s_%s"%(e.name, v) for v in e.values], 'NUM_%sS'%e.name): print(l)
		print("")

	for e in network.Flags:
		for l in create_flags_table(["%s_%s" % (e.name, v) for v in e.values]): print(l)
		print("")

	for l in create_enum_table(["NETOBJ_INVALID"]+[o.enum_name for o in network.Objects], "NUM_NETOBJTYPES"): print(l)
	print("")
	for l in create_enum_table(["NETMSG_INVALID"]+[o.enum_name for o in network.Messages], "NUM_NETMSGTYPES"): print(l)
	print("")

	for item in network.Objects + network.Messages:
		for line in item.emit_declaration():
			print(line)
		print("")

	EmitEnum(["SOUND_%s"%i.name.value.upper() for i in content.container.sounds.items], "NUM_SOUNDS")
	EmitEnum(["WEAPON_%s"%i.name.value.upper() for i in content.container.weapons.id.items], "NUM_WEAPONS")

	print("""

class CNetObjHandler
{
	const char *m_pMsgFailedOn;
	char m_aMsgData[1024];
	const char *m_pObjFailedOn;
	int m_NumObjFailures;
	bool CheckInt(const char *pErrorMsg, int Value, int Min, int Max);
	bool CheckFlag(const char *pErrorMsg, int Value, int Mask);

	static const char *ms_apObjNames[];
	static int ms_aObjSizes[];
	static const char *ms_apMsgNames[];

public:
	CNetObjHandler();

	int ValidateObj(int Type, const void *pData, int Size);
	const char *GetObjName(int Type) const;
	int GetObjSize(int Type) const;
	const char *FailedObjOn() const;
	int NumObjFailures() const;
	
	const char *GetMsgName(int Type) const;
	void *SecureUnpackMsg(int Type, CUnpacker *pUnpacker);
	const char *FailedMsgOn() const;
};

""")

	print("#endif // GAME_GENERATED_PROTOCOL_H")


if gen_network_source:
	# create names
	lines = []

	lines += ['#include <engine/shared/protocol.h>']
	lines += ['#include <engine/message.h>']
	lines += ['#include "protocol.h"']

	lines += ['CNetObjHandler::CNetObjHandler()']
	lines += ['{']
	lines += ['\tm_pMsgFailedOn = "";']
	lines += ['\tm_pObjFailedOn = "";']
	lines += ['\tm_NumObjFailures = 0;']
	lines += ['}']
	lines += ['']
	lines += ['const char *CNetObjHandler::FailedObjOn() const { return m_pObjFailedOn; }']
	lines += ['int CNetObjHandler::NumObjFailures() const { return m_NumObjFailures; }']
	lines += ['const char *CNetObjHandler::FailedMsgOn() const { return m_pMsgFailedOn; }']
	lines += ['']
	lines += ['']
	lines += ['']
	lines += ['']

	lines += ['static const int max_int = 0x7fffffff;']
	lines += ['']

	lines += ['bool CNetObjHandler::CheckInt(const char *pErrorMsg, int Value, int Min, int Max)']
	lines += ['{']
	lines += ['\tif(Value < Min || Value > Max) { m_pObjFailedOn = pErrorMsg; m_NumObjFailures++; return false; }']
	lines += ['\treturn true;']
	lines += ['}']
	lines += ['']

	lines += ['bool CNetObjHandler::CheckFlag(const char *pErrorMsg, int Value, int Mask)']
	lines += ['{']
	lines += ['\tif((Value&Mask) != Value) { m_pObjFailedOn = pErrorMsg; m_NumObjFailures++; return false; }']
	lines += ['\treturn true;']
	lines += ['}']
	lines += ['']

	lines += ["const char *CNetObjHandler::ms_apObjNames[] = {"]
	lines += ['\t"invalid",']
	lines += ['\t"%s",' % o.name for o in network.Objects]
	lines += ['\t""', "};", ""]

	lines += ["int CNetObjHandler::ms_aObjSizes[] = {"]
	lines += ['\t0,']
	lines += ['\tsizeof(%s),' % o.struct_name for o in network.Objects]
	lines += ['\t0', "};", ""]


	lines += ['const char *CNetObjHandler::ms_apMsgNames[] = {']
	lines += ['\t"invalid",']
	for msg in network.Messages:
		lines += ['\t"%s",' % msg.name]
	lines += ['\t""']
	lines += ['};']
	lines += ['']

	lines += ['const char *CNetObjHandler::GetObjName(int Type) const']
	lines += ['{']
	lines += ['\tif(Type < 0 || Type >= NUM_NETOBJTYPES) return "(out of range)";']
	lines += ['\treturn ms_apObjNames[Type];']
	lines += ['};']
	lines += ['']

	lines += ['int CNetObjHandler::GetObjSize(int Type) const']
	lines += ['{']
	lines += ['\tif(Type < 0 || Type >= NUM_NETOBJTYPES) return 0;']
	lines += ['\treturn ms_aObjSizes[Type];']
	lines += ['};']
	lines += ['']


	lines += ['const char *CNetObjHandler::GetMsgName(int Type) const']
	lines += ['{']
	lines += ['\tif(Type < 0 || Type >= NUM_NETMSGTYPES) return "(out of range)";']
	lines += ['\treturn ms_apMsgNames[Type];']
	lines += ['};']
	lines += ['']


	for l in lines:
		print(l)

	if 0:
		for item in network.Objects:
			for line in item.emit_validate():
				print(line)
			print("")

	# create validate tables
		lines = []
		lines += ['static int validate_invalid(void *data, int size) { return -1; }']
		lines += ["typedef int(*VALIDATEFUNC)(void *data, int size);"]
		lines += ["static VALIDATEFUNC validate_funcs[] = {"]
		lines += ['\tvalidate_invalid,']
		lines += ['\tvalidate_%s,' % o.name for o in network.Objects]
		lines += ["\t0x0", "};", ""]

		lines += ["int netobj_validate(int type, void *data, int size)"]
		lines += ["{"]
		lines += ["\tif(type < 0 || type >= NUM_NETOBJTYPES) return -1;"]
		lines += ["\treturn validate_funcs[type](data, size);"]
		lines += ["};", ""]

	lines = []
	lines += ['int CNetObjHandler::ValidateObj(int Type, const void *pData, int Size)']
	lines += ['{']
	lines += ['\tswitch(Type)']
	lines += ['\t{']

	for item in network.Objects:
		for line in item.emit_validate():
			lines += ["\t" + line]
		lines += ['\t']
	lines += ['\t}']
	lines += ['\treturn -1;']
	lines += ['};']
	lines += ['']

 #int Validate(int Type, void *pData, int Size);

	if 0:
		for item in network.Messages:
			for line in item.emit_unpack():
				print(line)
			print("")

		lines += ['static void *secure_unpack_invalid(CUnpacker *pUnpacker) { return 0; }']
		lines += ['typedef void *(*SECUREUNPACKFUNC)(CUnpacker *pUnpacker);']
		lines += ['static SECUREUNPACKFUNC secure_unpack_funcs[] = {']
		lines += ['\tsecure_unpack_invalid,']
		for msg in network.Messages:
			lines += ['\tsecure_unpack_%s,' % msg.name]
		lines += ['\t0x0']
		lines += ['};']

	#
	lines += ['void *CNetObjHandler::SecureUnpackMsg(int Type, CUnpacker *pUnpacker)']
	lines += ['{']
	lines += ['\tm_pMsgFailedOn = 0;']
	lines += ['\tm_pObjFailedOn = 0;']
	lines += ['\tswitch(Type)']
	lines += ['\t{']


	for item in network.Messages:
		for line in item.emit_unpack():
			lines += ["\t" + line]
		lines += ['\t']

	lines += ['\tdefault:']
	lines += ['\t\tm_pMsgFailedOn = "(type out of range)";']
	lines += ['\t\tbreak;']
	lines += ['\t}']
	lines += ['\t']
	lines += ['\tif(pUnpacker->Error())']
	lines += ['\t\tm_pMsgFailedOn = "(unpack error)";']
	lines += ['\t']
	lines += ['\tif(m_pMsgFailedOn || m_pObjFailedOn) {']
	lines += ['\t\tif(!m_pMsgFailedOn)']
	lines += ['\t\t\tm_pMsgFailedOn = "";']
	lines += ['\t\tif(!m_pObjFailedOn)']
	lines += ['\t\t\tm_pObjFailedOn = "";']
	lines += ['\t\treturn 0;']
	lines += ['\t}']
	lines += ['\tm_pMsgFailedOn = "";']
	lines += ['\tm_pObjFailedOn = "";']
	lines += ['\treturn m_aMsgData;']
	lines += ['};']
	lines += ['']


	for l in lines:
		print(l)

if gen_client_content_header or gen_server_content_header:
	print("#endif")



#from random import choice

import random
import os

masterservers = ["localhost 8300"]

maps = [
	["dm1", "dm2", "dm6"],
	["dm1", "dm2", "dm6"],
	["ctf1", "ctf2", "ctf3"],
]

servernames = [
	"%s playhouse",
	"%s own server",
]

nicks = []
for l in file("scripts/nicks.txt"):
	nicks += l.replace(":port80c.se.quakenet.org 353 matricks_ = #pcw :", "").strip().split()
inick = 0

def get_nick():
	global inick, nicks
	inick = (inick+1)%len(nicks)
	return nicks[inick].replace("`", "\`")
	
for s in xrange(0, 350):
	cmd = "./fake_server_d_d "
	cmd += '-n "%s" ' % (random.choice(servernames) % get_nick())
	for m in masterservers:
		cmd += '-m %s '%m
	
	max = random.randint(2, 16)
	cmd += "-x %d " % max
	
	t = random.randint(0, 2)
	
	cmd += '-a "%s" ' % random.choice(maps[t])
	cmd += '-g %d ' % random.randint(0, 100)
	cmd += '-t %d ' % t # dm, tdm, ctf
	cmd += "-f %d " % random.randint(0, 1) # password protected
		
	for p in xrange(0, random.randint(0, max)):
		cmd += '-p "%s" %d ' % (get_nick(), random.randint(0, 20))
	
	print cmd
	os.popen2(cmd)




import sys, os

line_count = 0

class variable:
	name = "unknown"
	def __init__(self, args, name):
		global line_count
		self.name = name
		self.line = line_count
	def emit_declaration(self):
		return ["\tint %s;" % self.name]
	def linedef(self):
		return "#line %d" % self.line
	def emit_secure(self, parent):
		return []
	def emit_unpack(self):
		return ["msg.%s = msg_unpack_int();" % self.name]
	def emit_unpack_check(self):
		return []
	def emit_pack(self):
		return ["\t\tmsg_pack_int(%s);" % self.name]

class var_any(variable):
	def __init__(self, args, name):
		variable.__init__(self, args, name)

class var_range(variable):
	def __init__(self, args, name):
		variable.__init__(self, args, name)
		self.min = args[0]
		self.max = args[1]
	def emit_unpack_check(self):
		return ["if(msg.%s < %s || msg.%s > %s) { msg_failed_on = \"%s\"; return 0; }" % (self.name, self.min, self.name, self.max, self.name)]
	def emit_secure(self, parent):
		return [self.linedef(), "obj->%s = netobj_clamp_int(\"%s.%s\", obj->%s, %s, %s);" % (self.name, parent.name, self.name, self.name, self.min, self.max)]

class var_string(variable):
	def __init__(self, args, name):
		variable.__init__(self, args, name)

class var_string(variable):
	def __init__(self, args, name):
		variable.__init__(self, args, name)
	def emit_declaration(self):
		return ["\tconst char *%s;" % self.name]
	def emit_unpack(self):
		return ["msg.%s = msg_unpack_string();" % self.name]
	def emit_pack(self):
		return ["\t\tmsg_pack_string(%s, -1);" % self.name]

class object:
	def __init__(self, line):
		fields = line.split()
		self.name = fields[1]
		self.extends = None
		if len(fields) == 4 and fields[2] == "extends":
			self.extends = fields[3]
		self.enum_name = "NETOBJTYPE_%s" % self.name.upper()
		self.struct_name = "NETOBJ_%s" % self.name.upper()
		self.members = []
		
	def parse(self, lines):
		global line_count
		for index in xrange(0, len(lines)):
			line_count += 1
			line = lines[index]
			if not len(line):
				continue
				
			if line == "end":
				return lines[index+1:]
			else:
				# check for argument
				fields = line.split(")", 1)
				if len(fields) == 2:
					names = [line.strip() for line in fields[1].split(",")]
					l = fields[0].split("(", 1)
					type = l[0]
					args = [line.strip() for line in l[1].split(",")]
				else:
					l = fields[0].split(None, 1)
					type = l[0]
					args = []
					names = [line.strip() for line in l[1].split(",")]
					
				for name in names:
					create_string = 'var_%s(%s, "%s")' % (type, args, name)
					new_member = eval(create_string)
					self.members += [new_member]
				
		raise BaseException("Parse error")
	
	def emit_declaration(self):
		lines = []
		if self.extends:
			lines += ["struct %s : public NETOBJ_%s\n {" % (self.struct_name, self.extends.upper())]
		else:
			lines += ["struct %s\n {" % self.struct_name]
		for m in self.members:
			lines += m.emit_declaration()
		lines += ["};"]
		return lines

	def emit_secure(self):
		lines = []
		for m in self.members:
			lines += m.emit_secure(self)
		return lines

class message:
	def __init__(self, line):
		fields = line.split()
		self.name = fields[1]
		self.enum_name = "NETMSGTYPE_%s" % self.name.upper()
		self.struct_name = "NETMSG_%s" % self.name.upper()
		self.members = []
		
	def parse(self, lines):
		global line_count
		for index in xrange(0, len(lines)):
			line_count += 1
			line = lines[index]
			if not len(line):
				continue
				
			if line == "end":
				return lines[index+1:]
			else:
				# check for argument
				fields = line.split(")", 1)
				if len(fields) == 2:
					names = [line.strip() for line in fields[1].split(",")]
					l = fields[0].split("(", 1)
					type = l[0]
					args = [line.strip() for line in l[1].split(",")]
				else:
					l = fields[0].split(None, 1)
					type = l[0]
					args = []
					names = [line.strip() for line in l[1].split(",")]
					
				for name in names:
					create_string = 'var_%s(%s, "%s")' % (type, args, name)
					new_member = eval(create_string)
					self.members += [new_member]
				
		raise BaseException("Parse error")
	
	def emit_declaration(self):
		lines = []
		lines += ["struct %s\n {" % self.struct_name]
		for m in self.members:
			lines += m.emit_declaration()
		lines += ["\tvoid pack(int flags)"]
		lines += ["\t{"]
		lines += ["\t\tmsg_pack_start(%s, flags);" % self.enum_name]
		for m in self.members:
			lines += m.emit_pack()
		lines += ["\t\tmsg_pack_end();"]
		lines += ["\t}"]
		lines += ["};"]
		return lines

	def emit_unpack(self):
		lines = []
		for m in self.members:
			lines += m.emit_unpack()
		for m in self.members:
			lines += m.emit_unpack_check()
		return lines

	def emit_pack(self):
		lines = []
		for m in self.members:
			lines += m.emit_pack()
		return lines
		

class event(object):
	def __init__(self, line):
		object.__init__(self, line)
		self.enum_name = "NETEVENTTYPE_%s" % self.name.upper()
		self.struct_name = "NETEVENT_%s" % self.name.upper()

class raw_reader:
	def __init__(self):
		self.raw_lines = []
	def parse(self, lines):
		global line_count
		for index in xrange(0, len(lines)):
			line_count += 1
			line = lines[index]
			if not len(line):
				continue
				
			if line == "end":
				return lines[index+1:]
			else:
				self.raw_lines += [line]
				
		raise BaseException("Parse error")		

class proto:
	def __init__(self):
		self.objects = []
		self.messages = []
		self.source_raw = []
		self.header_raw = []

def load(filename):
	# read the file
	global line_count
	line_count = 0
	lines = [line.split("//", 2)[0].strip() for line in file(filename).readlines()]
	
	p = proto()

	while len(lines):
		line_count += 1
		line = lines[0]
		
		if not len(line):
			del lines[0]
			continue
			
		fields = line.split(None, 1)
		
		del lines[0]
		
		if fields[0] == "object":
			new_obj = object(line)
			lines = new_obj.parse(lines)
			p.objects += [new_obj]
		elif fields[0] == "message":
			new_msg = message(line)
			lines = new_msg.parse(lines)
			p.messages += [new_msg]
		elif fields[0] == "event":
			new_obj = event(line)
			lines = new_obj.parse(lines)
			p.objects += [new_obj]
		elif fields[0] == "raw_source":
			raw = raw_reader()
			lines = raw.parse(lines)
			p.source_raw += raw.raw_lines
		elif fields[0] == "raw_header":
			raw = raw_reader()
			lines = raw.parse(lines)
			p.header_raw += raw.raw_lines
		else:
			print "error, strange line:", line
			
	return p
	
def emit_header_file(f, p):
	for l in p.header_raw:
		print >>f, l
		
	if 1: # emit the enum table for objects
		print >>f, "enum {"
		print >>f, "\tNETOBJTYPE_INVALID=0,"
		for obj in p.objects:
			print >>f, "\t%s," % obj.enum_name
		print >>f, "\tNUM_NETOBJTYPES"
		print >>f, "};"
		print >>f, ""

	if 1: # emit the enum table for messages
		print >>f, "enum {"
		print >>f, "\tNETMSGTYPE_INVALID=0,"
		for msg in p.messages:
			print >>f, "\t%s," % msg.enum_name
		print >>f, "\tNUM_NETMSGTYPES"
		print >>f, "};"
		print >>f, ""

	print >>f, "int netobj_secure(int type, void *data, int size);"
	print >>f, "const char *netobj_get_name(int type);"
	print >>f, "int netobj_num_corrections();"
	print >>f, "const char *netobj_corrected_on();"
	print >>f, ""
	print >>f, "void *netmsg_secure_unpack(int type);"
	print >>f, "const char *netmsg_get_name(int type);"
	print >>f, "const char *netmsg_failed_on();"
	print >>f, ""

	for obj in p.objects:
		for l in obj.emit_declaration():
			print >>f, l
		print >>f, ""

	for msg in p.messages:
		for l in msg.emit_declaration():
			print >>f, l
		print >>f, ""
			
def emit_source_file(f, p, protofilename):
	print >>f, "#line 1 \"%s\"" % os.path.abspath(protofilename).replace("\\", "\\\\")
	
	for l in p.source_raw:
		print >>f, l

	print >>f, "const char *msg_failed_on = \"\";"
	print >>f, "const char *obj_corrected_on = \"\";"
	print >>f, "static int num_corrections = 0;"
	print >>f, "int netobj_num_corrections() { return num_corrections; }"
	print >>f, "const char *netobj_corrected_on() { return obj_corrected_on; }"
	print >>f, "const char *netmsg_failed_on() { return msg_failed_on; }"
	print >>f, ""
	print >>f, "static int netobj_clamp_int(const char *error_msg, int v, int min, int max)"
	print >>f, "{"
	print >>f, "\tif(v<min) { obj_corrected_on = error_msg; num_corrections++; return min; }"
	print >>f, "\tif(v>max) { obj_corrected_on = error_msg; num_corrections++; return max; }"
	print >>f, "\treturn v;"
	print >>f, "}"
	print >>f, ""
	
	if 1: # names
		print >>f, "static const char *object_names[] = {"
		print >>f, "\t" + '"invalid",'
		for obj in p.objects:
			print >>f, '\t"%s",' % obj.name
		print >>f, '\t""'
		print >>f, "};"
		print >>f, ""
		
	if 1: # secure functions
		print >>f, "static int secure_object_invalid(void *data, int size) { return 0; }"
		for obj in p.objects:
			print >>f, "static int secure_%s(void *data, int size)" % obj.name
			print >>f, "{"
			print >>f, "\t%s *obj = (%s *)data;" % (obj.struct_name, obj.struct_name)
			print >>f, "\t(void)obj;" # to get rid of "unused variable" warning
			print >>f, "\tif(size != sizeof(%s)) return -1;" % obj.struct_name
			if obj.extends:
				print >>f, "\tif(secure_%s(data, sizeof(NETOBJ_%s)) != 0) return -1;" % (obj.extends, obj.extends.upper())
				
			for l in obj.emit_secure():
				print >>f, "\t" + l
			print >>f, "\treturn 0;";
			print >>f, "}"
			print >>f, ""

	if 1: # secure function table
		print >>f, "typedef int(*SECUREFUNC)(void *data, int size);"
		print >>f, "static SECUREFUNC secure_funcs[] = {"
		print >>f, "\t" + 'secure_object_invalid,'
		for obj in p.objects:
			print >>f, "\tsecure_%s," % obj.name
		print >>f, "\t" + '0x0'
		print >>f, "};"
		print >>f, ""

	if 1:
		print >>f, "int netobj_secure(int type, void *data, int size)"
		print >>f, "{"
		print >>f, "\tif(type < 0 || type >= NUM_NETOBJTYPES) return -1;"
		print >>f, "\treturn secure_funcs[type](data, size);"
		print >>f, "};"
		print >>f, ""

	if 1:
		print >>f, "const char *netobj_get_name(int type)"
		print >>f, "{"
		print >>f, "\tif(type < 0 || type >= NUM_NETOBJTYPES) return \"(invalid)\";"
		print >>f, "\treturn object_names[type];"
		print >>f, "};"
		print >>f, ""
		
	if 1: # names
		print >>f, "static const char *message_names[] = {"
		print >>f, "\t" + '"invalid",'
		for msg in p.messages:
			print >>f, '\t"%s",' % msg.name
		print >>f, '\t""'
		print >>f, "};"
		print >>f, ""		

	if 1: # secure functions
		print >>f, "static void *secure_unpack_invalid() { return 0; }"
		for msg in p.messages:
			print >>f, "static void *secure_unpack_%s()" % msg.name
			print >>f, "{"
			print >>f, "\tstatic %s msg;" % msg.struct_name
			for l in msg.emit_unpack():
				print >>f, "\t" + l
			print >>f, "\treturn &msg;";
			print >>f, "}"
			print >>f, ""		

	if 1: # secure function table
		print >>f, "typedef void *(*SECUREUNPACKFUNC)();"
		print >>f, "static SECUREUNPACKFUNC secure_unpack_funcs[] = {"
		print >>f, "\t" + 'secure_unpack_invalid,'
		for msg in p.messages:
			print >>f, "\tsecure_unpack_%s," % msg.name
		print >>f, "\t" + '0x0'
		print >>f, "};"
		print >>f, ""
		
	if 1:
		print >>f, "void *netmsg_secure_unpack(int type)"
		print >>f, "{"
		print >>f, "\tvoid *msg;"
		print >>f, "\tmsg_failed_on = \"\";"
		print >>f, "\tif(type < 0 || type >= NUM_NETMSGTYPES) return 0;"
		print >>f, "\tmsg = secure_unpack_funcs[type]();"
		print >>f, "\tif(msg_unpack_error()) return 0;"
		print >>f, "\treturn msg;"
		print >>f, "};"
		print >>f, ""

	if 1:
		print >>f, "const char *netmsg_get_name(int type)"
		print >>f, "{"
		print >>f, "\tif(type < 0 || type >= NUM_NETMSGTYPES) return \"(invalid)\";"
		print >>f, "\treturn message_names[type];"
		print >>f, "};"
		print >>f, ""		
		
if sys.argv[1] == "header":
	p = load(sys.argv[2])
	emit_header_file(file(sys.argv[3], "w"), p)
elif sys.argv[1] == "source":
	p = load(sys.argv[2])
	emit_source_file(file(sys.argv[3], "w"), p, sys.argv[2])
else:
	print "invalid command"
	sys.exit(-1)



import struct, zlib, sys

class image:
	w = 0
	h = 0
	data = []

def read_tga(f):
	image = f.read()
	img_type = struct.unpack("<B", image[2:3])[0]
	img_bpp = struct.unpack("<B", image[16:17])[0]
	img_width = struct.unpack("<H", image[12:14])[0]
	img_height = struct.unpack("<H", image[14:16])[0]

	if img_type != 2 or img_bpp != 32:
		print "image must be a RGBA"

	start = 18+struct.unpack("B", image[0])[0]
	end = start + img_width*img_height*4
	image_data = image[start:end] # fetch raw data
	return image_data

def write_tga(f, w, h, bpp, data):
	f.write(struct.pack("<BBBHHBHHHHBB", 0, 0, 2, 0, 0, 0, 0, 0, w, h, bpp, 0) + data)





def load_png(f):
	def read(fmt): return struct.unpack("!"+fmt, f.read(struct.calcsize("!"+fmt)))
	def skip(count): f.read(count)

	# read signature
	if read("cccccccc") != ('\x89', 'P', 'N', 'G', '\r', '\n', '\x1a', '\n'):
		return 0

	# read chunks
	width = -1
	height = -1
	imagedata = ""
	while 1:
		size, id = read("I4s")
		if id == "IHDR": # read header
			width, height, bpp, colortype, compression, filter, interlace = read("IIBBBBB")
			if bpp != 8 or compression != 0 or filter != 0 or interlace != 0 or (colortype != 2 and colortype != 6):
				print "can't handle png of this type"
				print width, height, bpp, colortype, compression, filter, interlace
				return 0
			skip(4)
		elif id == "IDAT":
			imagedata += f.read(size)
			skip(4) # read data
		elif id == "IEND":
			break # we are done! \o/
		else:
			skip(size+4) # skip unknown chunks

	# decompress image data
	rawdata = map(ord, zlib.decompress(imagedata))
	
	# apply per scanline filters
	pitch = width*4+1
	bpp = 4
	imgdata = []
	prevline = [0 for x in xrange(0, (width+1)*bpp)]
	for y in xrange(0,height):
		filter = rawdata[pitch*y]
		pixeldata = rawdata[pitch*y+1:pitch*y+pitch]
		thisline = [0 for x in xrange(0,bpp)]
		def paeth(a, b, c):
			p = a + b - c
			pa = abs(p - a)
			pb = abs(p - b)
			pc = abs(p - c)
			if pa <= pb and pa <= pc:
				return a
			if pb <= pc:
				return b
			return c
		
		if filter == 0: f = lambda a,b,c: 0
		elif filter == 1: f = lambda a,b,c: a
		elif filter == 2: f = lambda a,b,c: b
		elif filter == 3: f = lambda a,b,c: (a+b)/2
		elif filter == 4: f = paeth
			
		for x in xrange(0, width*bpp):
			thisline += [(pixeldata[x] + f(thisline[x], prevline[x+bpp], prevline[x])) % 256]			
			
		prevline = thisline
		imgdata += thisline[4:]
	
	raw = ""
	for x in imgdata:
		raw += struct.pack("B", x)
		
	#print len(raw), width*height*4
	write_tga(file("test2.tga", "w"), width, height, 32, raw)
	return 0
	
load_png(file("butterfly2.png", "rb"))


import hashlib, sys, re

alphanum = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ_".encode()

def cstrip(lines):
	d = "".encode()
	for l in lines:
		l = re.sub("#.*".encode(), "".encode(), l)
		l = re.sub("//.*".encode(), "".encode(), l)
		d += l + " ".encode()
	d = re.sub("\/\*.*?\*/".encode(), "".encode(), d) # remove /* */ comments
	d = d.replace("\t".encode(), " ".encode()) # tab to space
	d = re.sub("  *".encode(), " ".encode(), d) # remove double spaces
	d = re.sub("".encode(), "".encode(), d) # remove /* */ comments

	d = d.strip()

	# this eats up cases like 'n {'
	i = 1
	while i < len(d)-2:
		if d[i:i + 1] == " ".encode():
			if not (d[i - 1:i] in alphanum and d[i+1:i + 2] in alphanum):
				d = d[:i] + d[i + 1:]
		i += 1
	return d

f = "".encode()
for filename in sys.argv[1:]:
	f += cstrip([l.strip() for l in open(filename, "rb")])

hash = hashlib.md5(f).hexdigest().lower()[16:]
#TODO 0.8: improve nethash creation
if hash == "cb14bf6dc197d153":
	hash = "802f1be60a05665f"
print('#define GAME_NETVERSION_HASH "%s"' % hash)



import os, shutil, zipfile, sys

if len(sys.argv) <= 1:
	print "%s VERSION [SVN TREE]" % sys.argv[0]
	sys.exit(-1)

version = sys.argv[1]
svn_tree = "tags/release-%s" % version

if len(sys.argv) > 2:
	svn_tree = sys.argv[2]

# make clean
if 1:
	try: shutil.rmtree("srcwork")
	except: pass
	try: os.mkdir("srcwork")
	except: pass

root_dir = os.getcwd() + "/srcwork"

# change dir
os.chdir(root_dir)

# fix bam
#if 1:
#	os.system("svn export http://stalverk80.se/svn/bam bam")
#	z = zipfile.ZipFile("../bam.zip", "w")
#	for root, dirs, files in os.walk("bam"):
#		for f in files:
#			z.write(root+"/"+ f)
#	z.close()

if 1:
	os.system("svn export svn://svn.teeworlds.com/teeworlds/%s teeworlds" % svn_tree)
	os.chdir("teeworlds")
	os.system("python scripts/make_release.py %s src" % version)
	os.chdir(root_dir)
	for f in os.listdir("teeworlds"):
		if "teeworlds" in f and "src" in f and (".zip" in f or ".tar.gz" in f):
			shutil.copy("teeworlds/"+f, "../" + f)
